module (..., lube.seeall)

local button_animated = require "lube.item.button_animated"

-- copy references to all the on_activate, on_select, etc functions in simple_animated 
--lube.callback_copy( button_animated, _M ) 

local animation_name = "HUD_PIS_REMAP_PADDLE"
local bar_placeholder_part_name = "pis_remap_countdown"
local icon_part_name = "pis_remap_pup_icon"

defaults = function( pParms ) 
	parms = pParms or {}
	parms.is_selectable = true
	parms.registration = anchor.centre
	parms.origination = anchor.centre
	button_animated.defaults(pParms)
	return parms
end

function initialise(self, menu, parms)
	parms.skin = animation_name
	button_animated.initialise(self, menu, parms)
	self:set_origination(anchor.centre)
	self:set_selectable(true)
	local w,h = self:get_size()
	
	self.player_suffix = parms.player_suffix
	self.parent_node = parms.parent_node
	self.associated_node_id = parms.associated_node_id
	self.selection_time = parms.selection_time/1000	

	self.player:set_position( 0, 0 )
	
	local icon_part = self.player:get_part_by_name_recursive( icon_part_name )
	self.icon_graphic_item = icon_part:get_part_by_index(1)
	
	reset_slice(self, menu, parms)
	self.player:set_rotation(parms.angle)
	icon_part:set_rotation(-parms.angle)
	icon_part:lock_rotation(true)
	
	self:set_group(id.pup_shortcut_slice_group)
	
	self.on_message = on_message
	self.on_event = on_event
	self.on_select = on_select
	self.on_activate = on_activate
	self.on_highlight = on_highlight
	self.play_animation = self.play_animation
	self.on_deselect = on_deselect
	self.on_dehighlight = on_dehighlight
end

reset_slice = function ( self, menu, parms )
	self.doing_transition = false
	self.is_leaf = parms.is_leaf
	self.data_index = parms.data_index
	self.position_index = parms.position_index
	self.custom_data = parms.custom_data
	self.available = true --parms.available
	self.category_callout = parms.category_callout
	self.is_useful = parms.is_useful
	local icon_graphic = parms.custom_data

	if parms.custom_data then
		if self.is_leaf then
			icon_graphic = icon_graphic .. "\\".. icon_graphic
		else
			icon_graphic = "PIS_Catogory_Icons\\".. icon_graphic
		end
		self.icon_graphic_item:set_image(icon_graphic)
		self.icon_graphic_item:lock_size(false)
		self.icon_graphic_item:set_scale(1,1)
	else
		self.icon_graphic_item:set_scale(0,0)
		self.icon_graphic_item:lock_size(true)
	end

	setup_animation_names(self)

	if self.is_leaf then
		if self.custom_data then
			self:set_property_watch( "powerup_picked_at_least_once" .. self.player_suffix .. "[" .. self.custom_data .. "]" )
		end
	end
end

on_refresh = function ( self, menu )
	local property_watch = self:get_property_watch()
	if property_watch then
		local pickedup_at_least_once = properties[property_watch]
		if pickedup_at_least_once ~= nil then
			set_available(self, menu, pickedup_at_least_once)
		end
	end
end

set_available = function ( self, menu, pickedup_at_least_once )
	local was_available = self.available
	self.available = true -- pickedup_at_least_once
	if self.custom_data then
		if self.available then
			if not was_available then
				self.play_animation(self, "unavailable_to_available")
			end
		end		
	end
	setup_animation_names(self)
end

setup_animation_names = function ( self )
	if self.available then
		if self.is_leaf then
			self.select_section_name = "tier3_selected_start"
			self.end_selected_section_name = "tier3_selected"
			self.unselected_section_name = "tier3_unselected"
			self.on_push_section_name 			= "onPush"
			self.roll_over_section_name 		= "rollOver"
			self.roll_out_section_name 			= "rollOut"
			self.press_section_name 			= "press"
			self.release_section_name 			= "release"
			self.release_outside_section_name 	= "releaseOutside"
			self.drag_over_section_name 		= "dragOver"
			self.drag_out_section_name 			= "dragOut"
			self.mouse_over_section_name 		= "mouseOver"
			self.mouse_out_section_name 		= "mouseOut"
			self.sound_activate					= sounds.PIS_Select
		else
			self.select_section_name = "selected_start"
			self.end_selected_section_name = "selected_transition"
			self.unselected_section_name = "unselected"
			self.on_pop_section_name = "onPop"
			self.sound_activate			= sounds.PIS_Change_Screen
			if self.is_useful or not self.custom_data then
				self.on_push_section_name 			= "onPush"
				self.roll_over_section_name 		= "rollOver"
				self.roll_out_section_name 			= "rollOut"
				self.press_section_name 			= "press"
				self.release_section_name 			= "release"
				self.release_outside_section_name 	= "releaseOutside"
				self.drag_over_section_name 		= "dragOver"
				self.drag_out_section_name 			= "dragOut"
				self.mouse_over_section_name 		= "mouseOver"
				self.mouse_out_section_name 		= "mouseOut"
			else
				self.on_push_section_name 			= "empty_onPush"
				self.roll_over_section_name 		= "empty_rollOver"
				self.roll_out_section_name 			= "empty_rollOut"
				self.press_section_name 			= "empty_press"
				self.release_section_name 			= "empty_release"
				self.release_outside_section_name 	= "empty_releaseOutside"
				self.drag_over_section_name 		= "empty_dragOut"
				self.drag_out_section_name 			= "empty_dragOver"
				self.mouse_over_section_name 		= "empty_mouseOver"
				self.mouse_out_section_name 		= "empty_mouseOut"
			end			
		end
	else
		self.on_push_section_name 			= "unavailable_onPush"
		self.on_pop_section_name 			= "unavailable_onPop"
		self.roll_over_section_name 		= "unavailable_rollOver"
		self.roll_out_section_name 			= "unavailable_rollOut"
		self.press_section_name 			= "unavailable_press"
		self.release_section_name 			= "unavailable_release"
		self.release_outside_section_name 	= "unavailable_releaseOutside"
		self.drag_over_section_name 		= "unavailable_dragOver"
		self.drag_out_section_name 			= "unavailable_dragOut"
		self.mouse_over_section_name 		= "unavailable_mouseOver"
		self.mouse_out_section_name 		= "unavailable_mouseOut"
		self.sound_activate					= sounds.PIS_Invalid_Not_Available
	end
end

update_visibility = function ( self, menu )
	
	self.play_animation(self, self.on_push_section_name)
	if self.custom_data then
		self:enable()
	else
		self:disable()
	end
end

get_is_leaf = function ( self )
	return self.is_leaf
end

function on_message( self, menu, msg_id, parms )
	if msg_id == message.play then
		if parms.animation then
			self.play_animation( self, parms.animation )
		end
	elseif msg_id == message.set_value then
		if parms.player_x then
			self.player:set_position_x( parms.player_x )
		end
		if parms.player_y then
			self.player:set_position_y( parms.player_y )
		end
	elseif msg_id == message.set_text then
		if self.anim_part_text then
			f.set_text( self, self.anim_part_text, parms.text  )
		end
	elseif msg_id == message.enable then
		self:enable()
	elseif msg_id == message.disable then
		self:disable()
	elseif msg_id == message.set_custom_data then
		self.custom_data = msg_parms.custom_data
	elseif msg_id == message.get_custom_data then
		return self.custom_data
	end

	if msg_id == message.set_value then
		if parms.reset_slice then
			if parms.after_animation then
				self.postponed_reset_animation = parms.after_animation
				self.postponed_reset_parms = parms.reset_slice
			else
				reset_slice(self, menu, parms.reset_slice)
			end
		end
		if parms.set_custom_data then
			self.custom_data = parms.set_custom_data
		end
		if parms.update_visibility ~= nil then
			update_visibility( self, menu)
		end	
		if parms.call_back_on_end then
			self.call_back_on_end_animation_parms = parms.call_back_on_end
		end
		if parms.end_selection ~= nil then
			self.play_animation(self, self.end_selected_section_name)
			self.doing_transition = true
			self.parent_node:send_message(message.set_value, { start_transition = true})
		end
		if parms.do_pop ~= nil then
			self.play_animation(self, self.on_pop_section_name)
			self.doing_transition = true
		end
		if parms.highlight then
			if self.custom_data then
				if parms.highlight == true then
					self.play_animation( self, self.mouse_over_section_name )
				else
					self.play_animation( self, self.mouse_out_section_name )
				end
			end
		end
		if parms.activate then
			if self.custom_data then
				if parms.activate == true then
					on_activate(self, menu)
				end
			end
		end
	end
	if msg_id == message.get_value then
		if parms.get_is_active then
			return self.custom_data ~= nil
		end
		if parms.get_custom_data then
			return self.custom_data
		end
		if parms.get_is_leaf then
			return self.is_leaf
		end
		if parms.get_associated_node_id then
			return self.associated_node_id
		end
		if parms.get_powerup_details then
			if self.custom_data then
				if self.is_leaf then
					if power_up_shortcut_lua_hud then
						local callout, cost, quantity, nature = power_up_shortcut_lua_hud:get_powerup_details(self.custom_data)	
						local main_text = ""
						if self.available then
							main_text = txt[callout]
						else
							main_text = txt.UI_LOCKED_UCASE
							cost = ""
						end
						return main_text, cost, quantity, nature
					else
						local callout = "!!!!" .. self.custom_data .. "!!!!"
						local cost = 123445678
						local quantity = 87654321
						local nature = 0
						return callout, cost, quantity, nature
					end
				else
					return self.category_callout
				end
			else
				return nil
			end
		end
		if parms.get_data_index then
			return self.data_index
		end
		if parms.get_is_available then
			return self.available
		end
	end
end

on_event = function( self, menu, part, evt, evt_data )
	if evt_data == self.on_pop_section_name or evt_data == self.end_selected_section_name then
		self.doing_transition = false
	end
	if ( evt == "animation_stop" ) and ( evt_data == self.release_section_name ) then 
		menu:item_event( self, event.click )
		if self.available then
			self.parent_node:send_message(message.set_value, { start_current_slice_selection = self.position_index})
		else
			self.doing_transition = false		
		end
	end 
	--if ( evt == "animation_stop" ) and ( evt_data == self.release_section_name ) then 
	--	menu:item_event( self, event.click )
	--end 

	if ( evt == "animation_stop" ) then
		if self.custom_data then
			if evt_data == self.end_selected_section_name then
				if not self.is_leaf then
					-- we've completed the selection of the current category
					-- tell the parent node to show the category that I represent!!!
					self.parent_node:send_message(message.set_value, { activate_node = self.custom_data})
				end
				--self.play_animation( self, self.mouse_over_section_name )
			end
			if self.call_back_on_end_animation_parms then
				self.call_back_on_end_animation_parms.animation_id = evt_data
				self.call_back_on_end_animation_parms.custom_data = self.custom_data
				self.parent_node:send_message(message.set_value, { on_end_animation_call_back = self.call_back_on_end_animation_parms})
				self.call_back_on_end_animation_parms = nil
			end
			if evt_data == self.postponed_reset_animation then
				if self.postponed_reset_parms then
					-- we requested a reset after this animation using the data in self.postponed_reset_parms
					reset_slice(self, menu, self.postponed_reset_parms)
				end
			end
		end
	end
end

on_select = function( self, menu) 
	if self.debug then 
		print( self, "on_select" ) 
	end
	
	if self.mouse_press then 
		self.play_animation( self, self.press_section_name )
		sound:play( sounds.PIS_Change_Screen )
	else
		self.play_animation( self, self.roll_over_section_name )
	end
	menu:item_event( self, event.select )
end

on_deselect = function( self, menu)
	if not self.doing_transition then
		if self.debug then 
			print( self, "on_deselect" ) 
		end
		
		if self:is_highlighted() then 
			self.play_animation( self, self.mouse_over_section_name )
		else
			self.play_animation( self, self.roll_out_section_name )
		end
		menu:item_event( self, event.deselect )
	end
end

on_highlight = function( self, menu)
	if self.debug then 
		print( self, "on_highlight" ) 
	end
	
	if not self:is_selected() then 
		self.play_animation( self, self.mouse_over_section_name )
	end
	
	local position_index = self.position_index
	
	self.parent_node:send_message(message.set_value, { on_slice_highlight = position_index})
	menu:item_event( self, event.highlight )
end

on_dehighlight = function( self, menu)
	if not self.doing_transition then
		if self.debug then 
			print( self, "on_dehighlight" ) 
		end
		
		if not self:is_selected() then 
			self.play_animation( self, self.mouse_out_section_name )
		end
		
		menu:item_event( self, event.dehighlight )
	end
end

on_activate = function( self, menu)
	if self.debug then 
		print( self, "on_activate" ) 
	end
	
	self.play_animation( self, self.release_section_name )
	menu:wait_for_item_event( self, event.click )

	if self.sound_activate then
		sound:play( self.sound_activate )
	end
	
	if self.available then
		self.parent_node:send_message(message.set_value, { start_transition = true})
	end
end

_print = print 
print = function( self, ... )
	_print( self:get_root_id(), ... )
end

on_mouse_over = function( self, menu, is_over, is_mouse_down )
	if self.debug then 
		print( self, "on_mouse_over, over =", is_over, " down =", is_mouse_down ) 
	end
	if is_mouse_down and self:is_selected() then 
		if is_over then 
			self.is_mouse_over = true
			self.play_animation( self, self.drag_over_section_name )
		else 
			self.is_mouse_over = false
			self.play_animation( self, self.drag_out_section_name )
		end
	else
		if not is_over and self.mouse_press then
			self.play_animation( self, self.release_outside_section_name )
		end
	end
	self.mouse_press = is_mouse_down
end
