module( ..., lube.menu_logic )

local car_grid_logic	= require "lube.item.car_grid"
local playlist_manager	= require "frontend.playlist_manager"
local garage			= require "frontend.garage"
local garage_common		= require "frontend.garage_common"
local advanced_options	= require "frontend.advanced_options"
local progress_manager	= require "frontend.progress_manager"
local route_defaults 	= require "route_defaults"

local switch_time = 0.3 -- when you move grid position, how long the transition takes
local race_event
local race_name

start = function( menu )
	sound:play( sounds.start_activate )
	pre_start_set_cars( menu )
	properties["menu_return"] = "arcade"
	
	local game_type = game_rules[race_event.game_type].game_type
	local tutorial = "watched_"..game_type

	
	local career = require ( "career."..properties.career_current )
	local unlock_level = progress_manager.get_value( "current_level" )
	local data = career.playlists
	local ai_apo = garage_common.get_unlock_levels_for_playlist(data, unlock_level)
	setup:set_max_ai_apo( ai_apo["armour"], ai_apo["power"], ai_apo["offensive"] )

	if not properties[tutorial] then
		local tutorial_menu = (require ("frontend.tutorial_"..game_type))
		if tutorial_menu then
			tutorial_menu:send_message( message.set_value, { confirm = false, game_type = game_type } )
			ui:push( tutorial_menu )
		end
	elseif not properties.watched_pis and watched_all_game_mode_tutorials() then
		local tutorial_pis	= require "frontend.tutorial_pis"
		tutorial_pis:send_message( message.set_value, { confirm = false, pis = true } )
		ui:push( tutorial_pis )
	else
		ui:pop_all()
		properties.game_type_generic = game_rules[properties.game_type].game_type
		setup:end_setup( true )
	end
end

watched_all_game_mode_tutorials = function()
	if 	properties.watched_classic and properties.watched_car_crusher and properties.watched_checkpoint_stampede and
		properties.watched_death_race and properties.watched_fox_n_hounds and properties.watched_ped_chase then
		return true
	end
	
	return false
end

on_item_event = function( menu, item, evt )
	local item_id = item:get_item_id()
	
	if evt == event.click then
		if item_id == id.a_button then
			if not item.key_source then
				if menu.changing_car then
					menu.changing_car:send_message( message.slot_changing_car, {confirm=true} )
				else
					local selected = menu:get_selected_item()
					local selected_id = (selected and selected:get_item_id())
					if selected_id == id.the_grid then
						selected:send_message( message.activate, {} )
					elseif selected_id == id.start then
						menu:activate_selected_item()
					end
				end
			end
		elseif item_id == id.start then
			start( menu )
		elseif item_id == id.lb_button then
			switch_grid_slots( menu, true )
		elseif item_id == id.rb_button then
			switch_grid_slots( menu, false )
		elseif item_id == id.x_button then
			customise( menu )
		elseif item_id == id.y_button then
			if properties.car_selection ~= "random" then
				toggle_info( menu )
			end
		elseif item_id == id.b_button then
			if menu.changing_car then
				menu.changing_car:send_message( message.slot_changing_car, {cancel=true} )
			else
				ui:pop()
			end
		end
	elseif evt == event.select then
		set_button_bar( menu )
	end
	if item_id == id.the_grid then
		if evt == event.value_click then
			push_car_select( menu )
--		elseif evt == event.select then
--			menu:send_item_message( id.the_grid, message.grid_view_slot )
		elseif evt == event.value_change then
			check_bio_string( menu )
			set_button_bar( menu )
		end
	end
end

customise = function( menu )
	local item = menu:send_item_message( id.the_grid, message.get_selection, {} )
	local player_slot = menu:send_item_message( id.the_grid, message.get_player_slot, {} )
	
	if item and player_slot and item == player_slot then
		local car_name = player_slot:send_message( message.get_value, {car=true} )
		local car = car_lot[car_name]
		if car and #(car:get_unlocked_skins()) > 1 then		
			garage:send_message( message.set_value, { state = "skins", pop_return = true, info = { lobby_menu=menu, using_car_name=car_name, slot=player_slot, allow_apo=true } } )
			ui:push( garage )
		end
	end
end

set_ticker_bio = function( menu, car_name )
	if menu.ticker_message_bio then
		ticker_manager:release_message( menu.ticker_message_bio )
		menu.ticker_message_bio = nil
	end
	if car_name then
		menu.ticker_message_bio = ticker_manager:add_messages( true, true, txt["ui_fe_character_bio_"..car_name], 1 )
	end
end

-- sets the string in the ticker
check_bio_string = function( menu )
	local car_name
	if menu.show_info then
		local item = menu:send_item_message( id.the_grid, message.get_selection, {} )
		if item then
			car_name = item:send_message( message.get_value, {car=true} )
		end
	end
	
	if menu.bio_car ~= car_name then
		set_ticker_bio( menu, car_name )
		menu.bio_car = car_name
	end
end

-- Data in the slots needs to be set in the game, 
pre_start_set_cars = function( menu )
	if properties.car_selection == "random" then
		local owned_cars = car_lot:get_all_cars() --get_usable_cars("sp")
		local car = owned_cars[ui:random( #owned_cars )]
		car_lot:set_player_car( 1, 0, car:get_dev_name() )
		setup:set_opponents( car_lot:get_grid( properties.number_ai_cars, 0--[[TODO, number_of_parts]], car:get_dev_name() ) )
	else
		local opponents = {}
		for i,slot in ipairs( menu.slot_items) do
			local car, skin = slot:send_message( message.get_value, { car=true } )
			if slot.racer.rtype == "ai" then
				opponents[#opponents+1] = car_lot:get_skin( car, skin )
			else
				car_lot:set_player_car( 1, 0, car, skin )
			end
		end
		setup:set_opponents( opponents )
	end
end

is_skin_used = function( menu, except, pcar, pskin )
	for i,slot in ipairs( menu.slot_items ) do
		if slot ~= except then
			local car, skin = slot:send_message( message.get_value, {car=true} )
			if car == pcar and skin == pskin then
				return true
			end
		end
	end
	return false
end

get_unused_skin = function( menu, of_car, fallback_default )
	used_skins = {}
	for i,slot in ipairs( menu.slot_items ) do
		local car, skin = slot:send_message( message.get_value, {car=true} )
		if car == of_car then
			used_skins[skin] = true
		end
	end
	local skins = car_lot[of_car]:get_unlocked_skins()
	for key,value in ipairs(skins) do
		local skin_name = value:get_dev_name()
		if not used_skins[skin_name] then
			return skin_name
		end
	end
	if fallback_default then
		car_lot[of_car]:get_default_skin():get_dev_name()
	end
end

ai_avert_all_skins = function( menu, car )
	for i,slot in ipairs( menu.slot_items ) do
		local draw = true
		local cur_car, cur_skin = slot:send_message( message.get_value, {car=true} )
		if cur_car == car and slot.racer.rtype == "ai" and not slot.racer.fix_skin then
			local skin = get_unused_skin( menu, car )
			if skin and is_skin_used( menu, slot, cur_car, cur_skin ) then
				slot:send_message( message.set_value, { skin=skin } )
				draw = false -- the above line causes a draw
			end
		end
		if draw then slot:send_message( message.redraw ) end
	end
end

ai_match_car = function( menu, car )
	for i,slot in ipairs( menu.slot_items ) do
		if slot.racer.rtype == "ai" then
			slot:send_message( message.set_value, { car=car, dont_draw = true } )
		end
	end
	ai_avert_all_skins( menu, car )
end

-- Checks player1s first so opponents can be set from it if car selection is "hosts_choice", forces a draw of all slots under all circumstances
setup_all_players_cars = function( menu )
	local player1
	for i,slot in ipairs( menu.slot_items ) do
		if slot.racer.rtype == "player1" then
			player1 = slot
			break
		end
	end
	
	local car_sel = properties.car_selection
	if car_sel == "random" then
		for _,slot in ipairs( menu.slot_items ) do
			slot:send_message( message.set_value, { car=false, skin=false } )
		end
	elseif car_sel == "chosen" then
		local car_name, skin_name = car_lot:get_player_car( 1, 1 )
		if not car_lot:get_skin( car_name, skin_name ) then
			car_name = car_lot:get_default_car_dev_name()
			skin_name = car_lot:get_default_skin_dev_name()
		end
		player1:send_message( message.set_value, { car=car_name, skin=skin_name } )
		local opponents = car_lot:get_grid( properties.number_ai_cars, 0--[[TODO, number_of_parts]], car_name )
		local i = 1
		for _,slot in ipairs( menu.slot_items ) do
			if slot.racer.rtype == "ai" then
				slot:send_message( message.set_value, { car=opponents[i]:get_car_of_skin():get_dev_name(), skin=opponents[i]:get_dev_name() } )
				i = i+1
			end
		end
	-- else
		-- local car_name, skin_name = car_lot:get_player_car( 1, 1 ) -- ( player_index, 1=singleplayer )
		-- local skin = car_lot:get_skin( car_name, skin_name )
		-- if not skin or ( car_sel == "eagles_only" and not skin:get_car_of_skin():is_eagle() ) then
			-- car_name = car_lot:get_default_car_dev_name()
			-- skin_name = car_lot:get_default_skin_dev_name()
		-- end
		-- player1:send_message( message.set_value, { car=car_name, skin=skin_name } )
		-- ai_match_car( menu, car_name )
	end
end

on_push = function( menu )
	properties["menu_return"] = "arcade"
	properties.event_order = properties.profile_event_order
	properties.allow_loss_on_pup_buy = properties.arcade_allow_loss_on_pup_buy
	local playlist = save_manager:restore_table_from_profile( 1, "arcade_playlist" )
	if not properties.reentering_lobby then
		playlist_manager.set_event_order( #playlist )
	else
		playlist_manager.increment_counter()
	end
	menu:refresh()
end

clear_default_ticker = function( menu )
	if menu.ticker_token then
		ticker_manager:release_message( menu.ticker_token )
	end
end

on_pop = function( menu )
	menu:send_item_message( id.the_grid, message.clear_grid, {} )
	
	set_ticker_bio( menu, nil ) -- releases ticker message
	clear_default_ticker( menu )
end

on_refresh = function( menu )
-- set race event
-- tell the C code so it can decide opponents; retrieve opponents
-- fill grid
	rich_presence:set_next_presence(7)
	menu.grid_set = false -- Whether the client has set one of the cars on the grid, if they have, don't rebuild the grid when they change car
	menu.show_info = false
	menu.changing_car = false
	menu:get_item_by_id( id.start ):set_selectable( true )
	
	local playlist = save_manager:restore_table_from_profile( 1, "arcade_playlist" )
	race_event = playlist[playlist_manager.get_event_index()]
	
	fill_random()
	
	local game_type = game_rules[race_event.game_type].game_type

	properties.powerups = properties.profile_powerups
	properties.grid_start = properties["profile_grid_start_"..game_type]
	properties.pis_purchase = properties.profile_pis_purchase
	properties.point_stealing  = properties["profile_point_stealing_"..game_type] or false
	properties.starting_credit = properties.profile_starting_credit
	
	setupEvent( menu )

	local car_sel = properties.profile_car_selection
	properties.car_selection = (car_sel == "hosts_choice") and "chosen" or car_sel
	
	properties.number_ai_cars = properties.ai_cars and 5 or 0
	properties.player1_slot = math.max( math.min( math.floor(properties.player1_slot), properties.number_ai_cars + 1 ), 1 ) -- un-float and make sure slot is small enough, then big enough
	
	menu.grid_item = menu:get_item_by_id(id.the_grid)
	menu.slot_items = menu:send_item_message( id.the_grid, message.make_the_grid, {count=properties.number_ai_cars + 1, menu=menu, lobby_type="arcade"} )
	
	for i,slot in ipairs( menu.slot_items ) do
		slot:send_message( message.assign_slot_player, { rtype = (i == properties.player1_slot) and "player1" or "ai", dont_draw=true } )
	end
	
	setup_all_players_cars( menu )
	
	race_name = race_config:get_race_name( race_event.environment, race_event.level, race_event.race )
	if race_name and race_name ~= "" then
		menu:send_item_message( id.event_button, message.set_text, { text = game_rules[race_event.game_type].game_rules_name.." // "..race_name } )
	end

	menu.bio_car = nil
	
	local mode_route_text = (txt.ui_fe_lobby_mode_route):gsub("%0", game_rules[race_event.game_type].game_rules_name):gsub("%1", race_name)
	
	-- settings ticker texts
	local separator = txt.fe_setting_value_separator
	local event_order_ticker = separator:gsub("%0", txt.ui_fe_event_order_ucase):gsub("%1", advanced_options.get_event_order_ticker( properties.event_order ) )
	local grid_start_ticker = separator:gsub("%0", txt.ui_fe_grid_start_ucase):gsub("%1", advanced_options.get_grid_start_ticker( properties.grid_start ) )
	local starting_credits_ticker = separator:gsub("%0", txt.ui_fe_starting_credit_ucase):gsub("%1", properties.starting_credit )
	local powerups_ticker = separator:gsub("%0", txt.ui_fe_powerups_ucase):gsub("%1", advanced_options.get_powerups_ticker( properties.powerups ) )
	local car_select_ticker = separator:gsub("%0", txt.ui_fe_grid_setup_ucase):gsub("%1", advanced_options.get_car_selection_ticker( properties.car_selection ) )
	
	local game_mode_flavour = game_rules[race_event.game_type].game_rules_explanation or ""
	
	clear_default_ticker( menu )
	menu.ticker_token = ticker_manager:add_messages( true, true, mode_route_text, 1, event_order_ticker, 1, grid_start_ticker, 1, starting_credits_ticker, 1, powerups_ticker, 1, car_select_ticker, 1, game_mode_flavour, 2 )
	
	set_button_bar( menu )
end

on_focus = function( menu, is_focus )
	rich_presence:set_next_presence(7)
end

toggle_info = function( menu )
	menu.show_info = not menu.show_info
	menu.grid_item.slots_menu:send_group_message( id.grid_slots, message.show_info )
	check_bio_string( menu )
end		

can_change_car = function( menu, item )
	return properties.car_selection == "chosen" or properties.car_selection == "hosts_choice"
end

push_car_select = function( menu )
	local item = menu:send_item_message( id.the_grid, message.get_selection, {} )
	if not item then return end
	local car_sel = properties.car_selection

	if car_sel == "chosen" or (item.racer.rtype == "player1" and car_sel == "hosts_choice") then
		if item.racer.rtype == "ai" then
			item:send_message( message.slot_changing_car, { cars=car_lot:get_viable_ai_cars() } )
			menu:get_item_by_id( id.start ):set_selectable( false )
			menu:send_item_message( id.the_grid, message.slot_changing_car, {} )
			menu.changing_car = item
			set_button_bar( menu )
			menu:send_item_message( id.the_grid, message.play, { dim = true } )
		else
			local car_name = item:send_message( message.get_value, {car=true} )
			local allowed_cars = car_lot:get_set_of_cars( { sort_by_tier=true, single_player=true } )
			garage:send_message( message.set_value, { info = { lobby_menu=menu, using_car_name=car_name, allowed_cars = allowed_cars, slot=item, allow_apo=true } } )
			ui:push( garage )
		end
	end
end

set_button_bar = function( menu )
	if not menu.slot_items then return end
	local buttons = {A=false,B=false,X=false,Y=false,RB=false,LB=false}

	if properties.car_selection ~= "random" then
		buttons.Y=txt.ui_fe_driver_info_ucase
	end
	if menu.changing_car then
		buttons.A = txt.ui_fe_confirm_ucase
		buttons.B = txt.ui_fe_cancel_ucase
	else
		buttons.B = txt.ui_fe_back_ucase
		if menu:get_item_by_id( id.the_grid ):is_selected() then
			buttons.A = txt.ui_fe_select_ucase	
			local item = menu:send_item_message( id.the_grid, message.get_selection, {} )
			local player_slot = menu:send_item_message( id.the_grid, message.get_player_slot, {} )
			
			if item and player_slot and item == player_slot then
				local car_name = item:send_message( message.get_value, {car=true} )
				local car = car_lot[car_name]
				if car and #(car:get_unlocked_skins()) > 1 then
					buttons.X = txt.ui_fe_customise_ucase
				end
			end
		else
			buttons.A = txt.ui_fe_start_race_ucase
		end

		if properties.player1_slot ~= 1 then
			buttons.LB = txt.ui_fe_move_up_grid_ucase
		end
		if properties.player1_slot ~= #menu.slot_items then
			buttons.RB = txt.ui_fe_move_down_grid_ucase
		end
	end
	
	menu:send_item_message( id.button_bar, message.set_value, buttons )
end	

switch_to_slot = function( menu, slot )
	
end

switch_grid_slots = function( menu, up )
	if menu.changing_car then return end
	local s1 = up and properties.player1_slot - 1 or properties.player1_slot + 1
	if s1 < 1 or s1 > #menu.slot_items then
		return
	end
	
	sound:play( sounds.grid_move ) 
	
	local i1 = menu.slot_items[s1]
	local i2 = menu.slot_items[properties.player1_slot]
	local p1x, p1y = car_grid_logic.calc_item_pos( menu.grid_item, s1 )
	local p2x, p2y = car_grid_logic.calc_item_pos( menu.grid_item, properties.player1_slot )
	
	i1:set_position(p2x, p2y)
	i2:set_position(p1x, p1y)

	properties.player1_slot = s1
	
	local racer = i2.racer
	i2:send_message( message.assign_slot_player, i1.racer )
	i1:send_message( message.assign_slot_player, racer )

	if i1:is_selected() then
		i2:select()
	elseif i2:is_selected() then
		i1:select()
	end

	i1:set_position(p1x, p1y, switch_time, transition.ease_in)
	i2:set_position(p2x, p2y, switch_time, transition.ease_in)
	
	set_button_bar( menu )
end

setupEvent = function( menu )
	properties.environment = race_event.environment
	properties.level = race_event.level
	properties.race = race_event.race
	properties.game_type = race_event.game_type
	
	local params = game_rules[properties.game_type].parameters or {}
	for i,value in ipairs(params) do
		game_rules_parameters[value.tag] = race_event[value.tag]
	end
	
	setupAdvancedOptions(menu)	
end

fill_random = function()
	local has_random
	if race_event.environment == "random" then
		has_random = true
		local route_list = race_config:get_config()
		local level_setup = require "frontend.level_setup"
		route_list = level_setup.get_route_list( route_list )
		
		local first_locked
		
		for i=1, #route_list do
			if route_list[i].locked then
				first_locked = i
				break
			end
		end
		
		local upper = first_locked and first_locked - 1 or #route_list

		local is_present = false
		repeat
			local random_index = race_config:get_non_repeat_random_race_index( upper )
			race_event.environment = route_list[random_index].environment
			race_event.level = route_list[random_index].level
			race_event.race = route_list[random_index].race
			
--			local computed_zad_name = "lev_"..race_event.environment.."_"..race_event.level..".zad"
			local computed_zad_name = "lev_"..race_event.level..".zad"
			is_present  = Application:IsZadDownloaded(computed_zad_name)
		until (is_present)
		
		--print( race_event.environment, race_event.level, race_event.race )
	end
	
	if race_event.game_type == "random" then
		has_random = true
		local value = progress_manager.get_random_mode( menu, false )
		race_event.game_type = value.game_mode
		--print( "race_event.game_type", race_event.game_type )
	end
	
	if has_random then
		-- set game mode parameters with game rule defaults
		local params = game_rules[race_event.game_type].parameters or {}
		
		for k, v in pairs( params ) do
			race_event[v.tag] = v.default
			--print( "race_event[v.tag]", v.tag, v.default )
		end
		
		-- attempt to set game mode parameters with race defaults	
		local defaults = route_defaults.lookup

		local environment = race_event.environment
		local level = race_event.level
		local route = race_event.race
		local game_type = race_event.game_type
		
		if params then
			if defaults[environment] then
				if defaults[environment][level] then
					if defaults[environment][level][route] then
						if defaults[environment][level][route][game_type] then
							for k, v in pairs( params ) do
								local value = defaults[environment][level][route][game_type][v.tag]
								if value then
									race_event[v.tag] = value
								end
							end
						end
					end
				end
			end
		end
	end
end

setupAdvancedOptions = function (menu)
	game_rules_parameters.powerups = properties.powerups
	game_rules_parameters.grid_start = properties.grid_start
end

on_message = function (menu, msg_id, msg_parms)
	if msg_id == message.car_selected then
		if msg_parms.on_grid then
			car_selected_on_grid( menu, msg_parms )
		else
			car_selected_in_menus( menu, msg_parms )
		end
	elseif msg_id == message.show_info then
		toggle_info( menu )
	end
end

car_selected_on_grid = function( menu, parms )
	menu:send_item_message( id.the_grid, message.car_selected )
	menu:get_item_by_id( id.start ):set_selectable( true )
	
	car_selected( menu, parms )
	
	menu.changing_car = false
	set_button_bar( menu )
	
	menu:send_item_message( id.the_grid, message.play, { un_dim = true } )
end

car_selected_in_menus = function( menu, parms )
	car_selected( menu, parms )
end

car_selected = function( menu, parms )	
	if parms.car then
		if not parms.skin then
			parms.skin = car_lot[parms.car]:get_default_skin():get_dev_name()
		end
		
		parms.slot = parms.slot or menu.changing_car
		if parms.slot.racer.rtype == "player1" then
			car_lot:set_player_car( 1, 1, parms.car, parms.skin, parms.wheel )
		else
			menu.grid_set = true
		end
		
		parms.slot:send_message( message.set_value, parms )
		
		if properties.car_selection == "hosts_choice" then
			ai_match_car( menu, parms.car )
		elseif properties.car_selection == "chosen" and parms.slot.racer.rtype == "player1" and not menu.grid_set then
--			setup_all_players_cars( menu )
		end
	elseif parms.skin then
		parms.fix_skin = true
		parms.slot:send_message( message.set_value, parms )
		if parms.slot.racer.rtype == "player1" then
			local car = parms.slot:send_message( message.get_value, {car=true} )
			car_lot:set_player_car( 1, 1, car, parms.skin )
			if not menu.grid_set then
--				setup_all_players_cars( menu )
			else
				ai_avert_all_skins( menu, car )
			end
		else
			menu.grid_set = true
		end
	end
end























