module( ..., lube.seeall)

local f = require "functions.functions"
local playlists = require "playlists"
local progress_manager = require "frontend.progress_manager"
local level_setup = require "frontend.level_setup"

local setup_menu
local index = nil
local new_event = nil
local game_type = nil
local select_change_ticker_delay = 500 -- in ms
local event_spamming_stopped = "event_spamming_stopped"
local is_locked
local unlock_chapter
local random_mode
local item_id
local last_item_id

b_button = function()
	return function( menu, item, evt )
		if evt == event.click then
			setup_menu:send_message( message.set_value, { cancel_changing_event = true } )
			ui:pop()
		end
	end
end

change_ticker = function( menu )
	if item_id ~= last_item_id then		
		ticker_manager:release_message( menu.ticker_message_rules )
		
		if is_locked then
			local text = txt.ui_fe_ticker_unlock_game_mode_ucase
			if unlock_chapter then
				text = text:gsub("%0", unlock_chapter )
			end
			menu.ticker_message_rules = ticker_manager:add_messages( true, true, text, 2 )
		else
			if random_mode then
				menu.ticker_message_rules = ticker_manager:add_messages( true, true, txt.ui_fe_random_explanation, 1 )
			else
				menu.ticker_message_rules = ticker_manager:add_messages( true, true, game_rules[game_type].game_rules_explanation, 2 )
			end
		end
		
		last_item_id = item_id
	end
end

selection_change = function( menu )
	menu:get_item_by_id( id.timer ).timer:set( select_change_ticker_delay, event_spamming_stopped )
end

game_type_list = function( level_menu )
	return function( menu, item, evt )		
		local selected_type = nil
		
		if evt == event.value_change then
			selected_type = item:send_message( message.get_selection )
		elseif evt == event.highlight then
			selected_type = item:send_message( message.get_highlight )
		elseif evt == event.dehighlight then
			selected_type = item:send_message( message.get_selection )
		end

		if selected_type then
			item_id = selected_type:get_item_id()

			local value = selected_type:send_message( message.get_value )
			game_type = value.game_mode
			unlock_chapter = value.unlock_chapter
			
			local networked_session = (menu.session_type ~= "arcade") and (menu.session_type ~= "career")
			if networked_session then
				if game_type ~= "classic" and game_type ~= "ped_chase" and game_type ~= "random" then
					game_type = game_type.."_multiplayer"
				end
			end
			
			is_locked = false --value.locked
			
			if setup_menu then
				if item_id == id.random then
					random_mode = true
				else
					random_mode = false
				end
				
				setup_menu:send_message( message.set_value, { change_image = true, icon = game_type } )
				selection_change( menu )
			end
		end
		
		if evt == event.value_click then
			local selected_type = item:send_message( message.get_selection )
			local value = selected_type:send_message( message.get_value )
			local game_type = value.game_mode
			unlock_chapter = value.unlock_chapter
			
			local networked_session = (menu.session_type ~= "arcade") and (menu.session_type ~= "career")
			if networked_session then
				if game_type ~= "classic" and game_type ~= "ped_chase" and game_type ~= "random" then
					game_type = game_type.."_multiplayer"
				end
			end
			
			local locked = value.locked
			
			if locked then
				return
			end
			
			local temp = {}

			for k, v in pairs( playlists.current[index] ) do
				if k == "environment" or k == "level" or k == "race" then
					temp[k] = v
				else
					if game_type ~= "random" then
						-- add any matching game rules params 
						local params = game_rules[game_type].parameters or {}
						
						for i=1, #params do
							if params[i].tag == k then
								-- new game mode uses the parameter, check the value is still valid
								local values = params[i].values
								local found = false
								
								for j=1, #values do
									if values[j] == v then
										found = true
										break
									end
								end
								
								if found then
									temp[k] = v
								else
									temp[k] = params[i].default
								end
								
								break
							end
						end
					end
				end
			end
			
			temp.game_type = game_type
			
			-- clear the old event and add the new event
			playlists.current[index] = {}
			
			for k, v in pairs( temp ) do
				playlists.current[index][k] = v
			end

			if new_event then
				setup_menu:send_message( message.set_value, { change_image = true, icon = game_type, play_anim = true, state = "level", new_event=true, } )
				level_menu:send_message( message.set_value, { index = index, new_event = true } )
				ui:push( level_menu )
			else
				ui:pop()
			end
		end
	end
end

pg_up_button = function()
	return function( menu, item, evt )
		if evt == event.click then
			menu:send_item_message( id.game_type_list, message.move, { page_up = true } )
		end
	end
end

pg_dn_button = function()
	return function( menu, item, evt ) 
		if evt == event.click then
			menu:send_item_message( id.game_type_list, message.move, { page_down = true } )
		end
	end
end

select_current = function( menu )	
	if playlists.current[index].game_type == "random" then
		menu:send_item_message( id.game_type_list, message.set_selection, { item_id = id.random } )
		menu:send_item_message( id.game_type_list, message.move, { into_view = true } )
	else
		game_type = string.lower( playlists.current[index].game_type )
		
		local list = menu:send_item_message( id.game_type_list, message.get_content, {} )
		local item_count = list:get_item_count()

		for i=1, item_count do
			local item = list:get_item_by_index( i )
			local value = (item:send_message( message.get_value, {} ))

			if value then
				local game_mode = value.game_mode
				
				if game_mode ==  game_rules[game_type].game_type then
					local item_id = item:get_item_id()
					menu:send_item_message( id.game_type_list, message.set_selection, { item_id = item_id } )
					menu:send_item_message( id.game_type_list, message.move, { into_view = true } )
				end
			end
		end
	end
end

do_select = function( menu )	
	if new_event then
		menu:send_item_message( id.game_type_list, message.set_selection, { first = true } )
	else
		select_current( menu )
	end
end

populate_grid = function( menu )
	local new_grid_items = {}
	
	local networked_session = (menu.session_type ~= "arcade") and (menu.session_type ~= "career")

	local mode_list = progress_manager.get_mode_list()
	
	local first_locked
	
	for i=1, #mode_list do
		if mode_list[i].locked then
			first_locked = i
			break
		end
	end

	local random = {}
	random.game_mode = "random"
	random.locked = false

	new_grid_items[#new_grid_items+1] = info_header_paged { text=txt.fe_random_ucase, }
	new_grid_items[#new_grid_items+1] = button_value_small_paged{ id=id.random, value=random, group=id.game_type_group, text=txt.ui_fe_random_game_mode_ucase, locked = false, skin="button_menu.bin",}
	
	new_grid_items[#new_grid_items+1] = info_header_paged { text=txt.ui_fe_game_modes_ucase, }
	
		for i=1, #mode_list do
			mode_list[i].locked = false
		end

		first_locked = nil
		
	
	for i=1, #mode_list do
		local game_mode = mode_list[i].game_mode
		
		if networked_session then
			if game_mode ~= "classic" and game_mode ~= "ped_chase" then
				game_mode = game_mode.."_multiplayer"
			end
		end
	
		if first_locked and i == first_locked then
			new_grid_items[#new_grid_items+1] = info_header_paged { text=txt.ui_locked_ucase, }
		end
	
		local value = game_rules[game_mode].game_type

		if value ~= "dev_race" and networked_session == game_rules[game_mode].network then
			if mode_list[i].locked then
				new_grid_items[#new_grid_items+1] = button_value_small_paged{ id=id[value], value=mode_list[i], group=id.game_type_group, text=game_rules[game_mode].game_rules_name, locked = mode_list[i].locked, skin="button_menu_unavailable.bin", }--highlights=false,}
			else
				new_grid_items[#new_grid_items+1] = button_value_small_paged{ id=id[value], value=mode_list[i], group=id.game_type_group, text=game_rules[game_mode].game_rules_name, locked = mode_list[i].locked, skin="button_menu.bin",}
			end
		end
	end

	menu:send_item_message( id.game_type_list, message.set_new_content, { content = new_grid_items, stripey = true } )
	
	do_select( menu )
end

on_refresh = function()
	return function( menu )
		populate_grid( menu )
	end
end

on_push = function( session_type )
	return function( menu )
		menu.session_type = session_type
		if not menu:get_item_by_id( id.timer ) then
			menu:add_items( {timer_item { id=id.timer }} )
		end
		menu:refresh()
	end
end

on_pop = function()
	return function( menu )
		ticker_manager:release_message( menu.ticker_message_rules )
		menu.ticker_message_rules = nil
		menu:get_item_by_id( id.timer ).timer:cancel()
		setup_menu:send_message( message.set_value, { change_image = true, icon = playlists.current[index].game_type } )
	end
end

set_button_bar = function( menu )
	local button_y
	if properties.in_network_session then
		button_y = false
	else
		button_y = txt.ui_fe_my_stats_ucase
	end
	menu:send_item_message( id.button_bar, message.set_value, {Y=button_y} )
end
		
on_focus = function()
	return function( menu, is_focus )
		if is_focus then
			set_button_bar( menu )
			if new_event then
				setup_menu:send_message( message.set_value, { play_anim = true, state="mode", new_event = true } )
			end
		end
	end
end

on_message = function()
	return function( menu, msg_id, parms )
		if msg_id == message.set_value then
			if parms.new_event then
				new_event = parms.new_event
			else
				new_event = nil
			end

			if parms.index then
				index = parms.index
			end
			
			if parms.setup_menu then
				setup_menu = parms.setup_menu
			end	
		elseif msg_id == message.timer_expired then
			if parms.evt_data == event_spamming_stopped then
				change_ticker( menu )
			end
		end
	end
end
