module( ..., lube.menu_logic )

local progress_manager	= require "frontend.progress_manager"
local garage_common		= require "frontend.garage_common"
local profile		= require "frontend.profile"
local dlg_ok 				= require "dialogue_boxes.ok"

local nav_trans_time = 7/30
local nav_trans = transition.linear
local info_time = 5/30 -- time it takes for the car to transition to it's position when the info is on the screen

local customise_pos_ratio_x = -0.4
local customise_pos_ratio_y = -0.3

local parts_pos_ratio_x = -0.4
local parts_pos_ratio_y = -0.3

local skin_pos_ratio_x = -0.5

local wheel_pos_ratio_x = -0.5

local info_scale = 0.8

local full_w
local full_h

local scaled_w
local scaled_h

local info_table = {}

local selected_car_name
local ticker_message_allowed_cars = nil

local state
local states = { garage = 1, customise = 2, parts_shop = 3, skins = 4, wheels = 5 }

local pop_return
local customise_return

local prev_skin_id
local prev_wheel_id

local skin_unlock_msg
local rim_unlock_msg

local prev_unlocked_skin
local prev_unlocked_rim
local last_skin_selection
local last_rim_selection

-- for updating ticker messages
local skin_type
local wheel_type

can_drive_car = function( car_name )
	if not car_name or not car_lot[car_name] then return false end
	
	if car_lot[car_name]:is_car_downloaded() == false then return false end
	
	if info_table.multiplayer then
		if not NetworkGame:GetIsSessionManager() then
			local car_selection = NetworkGame.car_selection -- "random", "chosen", "hosts_choice"
			
			if car_selection == "hosts_choice" then
				return false
			end
		end
		
		return info_table.using_car_name ~= car_name -- if you weren't allowed to use all the cars, allowed cars would be set, it's not though, because of above
	else -- otherwise it's career or freeplay, so you can only drive cars you owned (and some you don't own may be in the garage)
		local owned = true --car_lot[car_name]:is_owned()
		local not_using = info_table.using_car_name ~= car_name
		return owned and not_using
	end
end

on_event[id.a_button] = function( menu, item, evt )
	if state == states.garage then
		if not can_drive_car( selected_car_name ) then return end

		info_table.using_car_name = selected_car_name
		local car = car_lot[info_table.using_car_name]
		local skin_name = car:get_player_skin()
		sound:play( sounds.button_activate ) 	
		if info_table.lobby_menu then
			info_table.lobby_menu:send_message( message.car_selected, { on_grid=false, skin=skin_name, car=info_table.using_car_name, slot=info_table.slot } )
		end
		
		car_lot:set_player_car( 1, info_table.multiplayer and 2 or 1, car:get_dev_name() )

		set_button_bar( menu )
		set_ticker_equippedness( menu, selected_car_name )
		check_wheel( menu )
	else
		local f=lube.on_click_select()
		f( menu, item, evt )
	end
end

on_event[id.b_button] = function( menu, item, evt )
	if state == states.garage then
		ui:pop()
	elseif state == states.customise then
		if customise_return then
			customise_return = false
			state = states.garage
			ui:pop()
		else
			change_state( menu, states.garage )
		end
	elseif 	state == states.parts_shop or
			state == states.skins or
			state == states.wheels then
		if pop_return then
			pop_return = false
			state = states.garage
			ui:pop()
		else
			check_skin( menu )
			check_rim( menu )
			change_state( menu, states.customise )
		end
		
		save_manager:save()
	end
end

--on_event[id.y_button]		= lube.on_click_push( profile )

on_event[id.x_button] = function( menu, item, evt )
	if evt == event.click then
		if state == states.garage then
			if ttue or car_lot[selected_car_name]:is_owned() or info_table.multiplayer then
				change_state( menu, states.customise )
			end
		end
	end
end

on_event[id.spinner] = function( menu, item, evt )
	if evt == event.spinbox_left then
		navigate_cars( menu, -1 )
	elseif evt == event.spinbox_right then
		navigate_cars( menu, 1 )
	end
end

--car loading finish event
on_event[id.model1] = function( menu, item, evt )
	if evt == event.click then
		set_info(menu)
	end
end

on_event[id.model2] = function( menu, item, evt )
	if evt == event.click then
		set_info(menu)
	end
end

change_state = function( menu, new_state, just_pushed )
	local old_state = state
	state = new_state
	
	menu:send_item_message( id.armour_nav, message.play, { animation = "default" } )
	menu:send_item_message( id.power_nav, message.play, { animation = "default" } )
	menu:send_item_message( id.offensive_nav, message.play, { animation = "default" } )
	
	if state == states.garage then
		menu:show_group( id.garage )
		menu:show_group( id.info_stuff )
		if info_table.allow_apo then
			menu:show_group( id.apo )
		else
			menu:hide_group( id.apo )
		end
		
		menu:hide_group( id.customise )
		menu:hide_group( id.shop )
		menu:hide_group( id.armour )
		menu:hide_group( id.power )
		menu:hide_group( id.offensive )
		menu:hide_group( id.skins )
		menu:hide_group( id.wheels )
		
		transition_model( menu, "garage" )
		
		menu:send_item_message( id.background, message.set_value, { skin = "background_garage_01" } )
		
	elseif state == states.customise then
		menu:show_group( id.customise )
		if info_table.allow_apo then
			menu:enable_item( id.parts_shop )
		else
			menu:disable_item( id.parts_shop )
		end
		
		menu:hide_group( id.garage )
		menu:hide_group( id.info_stuff )
		
		if info_table.allow_apo then
			menu:show_group( id.apo )
		else
			menu:hide_group( id.apo )
		end
		
		menu:hide_group( id.shop )
		menu:hide_group( id.armour )
		menu:hide_group( id.power )
		menu:hide_group( id.offensive )
		menu:hide_group( id.skins )
		menu:hide_group( id.wheels )
		
		transition_model( menu, "customise" )
		
		if skin_unlock_msg then
			ticker_manager:release_message( skin_unlock_msg )
		end
		if rim_unlock_msg then
			ticker_manager:release_message( rim_unlock_msg )
		end
		
		menu:send_item_message( id.background, message.set_value, { skin = "background_garage_02" } )
		
	elseif state == states.parts_shop then
		menu:show_group( id.shop )
		-- if info_table.allow_apo then
			menu:show_group( id.apo )
		-- else
			-- menu:hide_group( id.apo )
		-- end
		
		menu:hide_group( id.garage )
		menu:hide_group( id.info_stuff )
		menu:hide_group( id.customise )
		menu:hide_group( id.skins )
		menu:hide_group( id.wheels )
		
		setup_apo( menu )
		
		transition_model( menu, "shop" )
		
		menu:send_item_message( id.background, message.set_value, { skin = "background_garage_02" } )
		
	elseif state == states.skins then
		skin_type = nil
		menu:show_group( id.skins )
		
		menu:hide_group( id.garage )
		menu:hide_group( id.info_stuff )
		menu:hide_group( id.apo )
		menu:hide_group( id.customise )
		menu:hide_group( id.shop )
		menu:hide_group( id.armour )
		menu:hide_group( id.power )
		menu:hide_group( id.offensive )
		menu:hide_group( id.wheels )
		
		transition_model( menu, "skins" )
		
		local car_name = get_car_name()
		
		original_slot_car = car_name
		if info_table.slot then
			original_slot_type = info_table.slot.racer.rtype
		end
		
		populate_skins( menu )
		menu:check_focus()
		
		prev_unlocked_skin = car_lot[selected_car_name]:get_player_skin()
		
		menu:send_item_message( id.background, message.set_value, { skin = "background_garage_02" } )

	elseif state == states.wheels then
		wheel_type = nil
		menu:show_group( id.wheels )
		
		menu:hide_group( id.garage )
		menu:hide_group( id.info_stuff )
		menu:hide_group( id.apo )
		menu:hide_group( id.customise )
		menu:hide_group( id.shop )
		menu:hide_group( id.armour )
		menu:hide_group( id.power )
		menu:hide_group( id.offensive )
		menu:hide_group( id.skins )
	
		transition_model( menu, "wheels" )
		
		populate_wheels( menu )
		menu:check_focus()
		
		prev_unlocked_rim = car_lot[selected_car_name]:get_player_wheel()
		
		menu:send_item_message( id.background, message.set_value, { skin = "background_garage_02" } )
	end
	
	if old_state ~= state then
		if old_state == states.parts_shop then
			set_part_ticker_text( menu, false )
		-- elseif old_state == states.wheels then
			-- set_rim_ticker_text( menu, false )
		-- elseif old_state == states.skins then
			-- set_skin_ticker_text( menu, false )
		end
	end
	
	set_button_bar( menu )
end

set_button_bar = function( menu )
	local buttons = {}
	
	if state == states.garage then
		if can_drive_car( selected_car_name ) then
			buttons.A = txt.ui_fe_get_in_ucase
		else
			buttons.A = false
		end
		
		buttons.B = txt.ui_fe_leave_garage_ucase
		
		if (car_lot[selected_car_name] and car_lot[selected_car_name]:is_owned() and car_lot[selected_car_name]:is_car_downloaded() )or info_table.multiplayer or true then
			buttons.X = txt.ui_fe_customise_ucase
		else
			buttons.X = false
		end
		
		--buttons.Y = txt.ui_fe_my_stats_ucase
		buttons.LT = false

	elseif state == states.customise then
		buttons.A = txt.ui_fe_select_ucase
		buttons.B = txt.ui_fe_back_ucase
		buttons.Y = false
		buttons.X = false		
		buttons.RT = false
		buttons.LT = false
		
	elseif state == states.parts_shop then
		buttons.A=txt.ui_fe_select_ucase
		buttons.B=txt.ui_fe_back_ucase
		buttons.Y = false
		buttons.X = false
		buttons.RT = false
		buttons.LT = false
	elseif state == states.skins then
		buttons.A=txt.ui_fe_select_ucase		
		buttons.B=txt.ui_fe_back_ucase
		buttons.Y = false
		buttons.X = false
		
	elseif state == states.wheels then
		buttons.A=txt.ui_fe_select_ucase
		buttons.B=txt.ui_fe_back_ucase
		buttons.Y = false
		buttons.X = false
	end

	menu:send_item_message( id.button_bar, message.set_value, buttons )
end

navigate_cars = function( menu, dir )
	selected_car_name = menu:send_item_message( id.spinner, message.get_value, {} )
	car_lot:set_garage_car( selected_car_name )
	
	local skin_name = "default"
	local wheel_name = "default"
	-- get the player's chosen skin/wheel for the car
	if car_lot[selected_car_name] then
		 skin_name = car_lot[selected_car_name]:get_player_skin()
		 wheel_name = car_lot[selected_car_name]:get_player_wheel()
		 set_info( menu )
	end
	
	local old_model = menu.focus_model
	local new_model = (menu.focus_model == menu.model1 and menu.model2 or menu.model1)
	menu.focus_model = new_model
	set_lit( menu )
	sound:play( sounds.button_activate ) 
	old_model:send_message( message.set_value, { disable_light = true } )
	new_model:send_message( message.set_value, { car_name = selected_car_name, skin_name = skin_name, wheel_name = wheel_name } )
	old_model:set_position_x( -dir*menu.sw, nav_trans_time, nav_trans )
	old_model:set_alpha( 0, nav_trans_time, nav_trans )
	new_model:set_position_x( dir*menu.sw )
	new_model:set_position_x( 0, nav_trans_time, nav_trans )	
	new_model:set_alpha( 255, nav_trans_time, nav_trans )

	update_apo( menu )
	
	set_button_bar( menu )
end

check_wheel = function( menu )
	local wheel = menu:send_item_message( id.wheel, message.get_value )
	local lock = menu:get_item_by_id( id.lock )
	
	if info_table.using_car_name == selected_car_name then
		wheel:set_alpha( 255, 0.2, transition.linear )
		lock:set_alpha( 0, 0.2, transition.linear )
	else
		wheel:set_alpha( 0, 0.2, transition.linear )

		if not info_table.multiplayer then
			if car_lot[selected_car_name]:is_owned() or true then 
				lock:set_alpha( 0, 0.2, transition.linear )
			else
				lock:set_alpha( 255, 0.2, transition.linear )
			end
		end
	end
end

set_ticker_bio = function( menu, car_name )
	local car = car_lot[car_name]
	if menu.bio_car ~= car then
		ticker_manager:release_message( menu.ticker_message_bio )
		menu.ticker_message_bio = nil
	end
	
	menu.bio_car = car
	if car then
		menu.ticker_message_bio = ticker_manager:add_messages( true, true, car:get_bio(), 1 )
	end
end

set_ticker_equippedness = function( menu, car_name )
	local new_message
	local icon = 1
	local car = car_lot[car_name]
	
	if car then
		if info_table.using_car_name == car_name then
			new_message = txt.ui_fe_current_vehicle_ucase
			icon = 12
		elseif can_drive_car( car_name ) then
			new_message = txt.ui_fe_get_in_vehicle_ucase
			icon = 12
		elseif info_table.multiplayer then
			if not NetworkGame:GetIsSessionManager() then
				local car_selection = NetworkGame.car_selection -- "random", "chosen", "hosts_choice"
		
				if car_selection == "hosts_choice" then
					new_message = txt.ui_fe_hosts_choice_ucase
					icon = 13
				end
			end
		end
		
		if not new_message then
			local unlock_node = car:get_unlock_node()
			
			if unlock_node >= 1 then
				new_message = txt.ui_fe_ticker_garage_steal_ucase
				new_message = new_message:gsub( "%0", unlock_node )
				icon = 2
			end
		end
	end
	
	if menu.equippedness ~= new_message then
		if menu.ticker_message_equippedness then
			ticker_manager:release_message( menu.ticker_message_equippedness )
			menu.ticker_message_equippedness = nil
		end
		
		menu.equippedness = new_message
		
		if new_message then
			menu.ticker_message_equippedness = ticker_manager:add_messages( true, true, new_message, icon )
		end
	end
end

-- set the character info in the driver info and the driver bio in the ticker
set_info = function( menu )
	local car = car_lot[selected_car_name]
	menu:send_item_message( id.info, message.set_values, {character=car:get_ai_driver_name(),
																			car_id=selected_car_name,
																			portrait=car:get_portrait(),
																			} )

	if --[[car_lot[selected_car_name]:is_seen() or]] car_lot[selected_car_name]:is_owned() or true or info_table.multiplayer --[[or ( properties.menu_return == "career" and car_lot:is_stealworthy( selected_car_name ) )]] then
		menu:send_item_message( id.info, message.play, {animation="enabled"} )
	else
		menu:send_item_message( id.info, message.play, {animation="not_available"} )
	end

	check_wheel( menu )

	set_ticker_bio( menu, selected_car_name )
	set_ticker_equippedness( menu, selected_car_name )
end

render_state = function( menu, instant )
	
	
	if info_table.allow_apo then
		menu:show_group( id.apo )
	else
		menu:hide_group( id.apo )
	end
	transition_model( menu, "garage" )
end

transition_model = function( menu )
	if state == states.garage then
		menu.focus_model:send_message( message.set_value, { scene = "garage" } )
		set_scaled_models( menu, 0, 0, full_w, full_h )

	elseif state == states.customise then
		menu.focus_model:send_message( message.set_value, { scene = "customise" } )
		set_scaled_models( menu, menu.sw/2*customise_pos_ratio_x, menu.sh/2*customise_pos_ratio_y )
		
	elseif state == states.parts_shop then
		menu.focus_model:send_message( message.set_value, { scene = "parts_shop" } )
		set_scaled_models( menu, menu.sw/2*parts_pos_ratio_x, menu.sh/2*parts_pos_ratio_y )
		
	elseif state == states.skins then
		menu.focus_model:send_message( message.set_value, { scene = "skins" } )
		set_scaled_models( menu, menu.sw/2*skin_pos_ratio_x, 0 )

	elseif state == states.wheels then
		menu.focus_model:send_message( message.set_value, { scene = "wheels" } )
		set_scaled_models( menu, menu.sw/2*wheel_pos_ratio_x, 0 )
	end
end

set_scaled_models = function( menu, x, y, w, h )
	local non_focus = (menu.focus_model==menu.model1 and menu.model2 or menu.model1)
	local ttime = (not instant) and info_time or 0
	local w = w or scaled_w
	local h = h or scaled_h
	menu.focus_model:set_position_x( x, ttime, transition.ease_in )
	menu.focus_model:set_position_y( y, ttime, transition.ease_in )
	menu.focus_model:set_size( w, h, ttime, transition.ease_in )
	menu.focus_model.i_car:set_size( w, h, ttime, transition.ease_in )
	non_focus:set_size( w, h, ttime, transition.ease_in )
	non_focus.i_car:set_size( w, h, ttime, transition.ease_in )
end

update_apo = function( menu )
	local car_dev_name = menu:send_item_message( id.spinner, message.get_value, {} )

	local armour = menu:get_item_by_id( id.armour )
	local power = menu:get_item_by_id( id.power )
	local offensive = menu:get_item_by_id( id.offensive )
	
	local armour_nav = menu:get_item_by_id( id.armour_nav )
	local power_nav = menu:get_item_by_id( id.power_nav )
	local offensive_nav = menu:get_item_by_id( id.offensive_nav )
	
	if true or car_lot[car_dev_name] and car_lot[car_dev_name]:is_owned() and not info_table.multiplayer then
		local armour_stat = car_lot[car_dev_name]:get_equipped_armour()
		local power_stat = car_lot[car_dev_name]:get_equipped_power()
		local offensive_stat = car_lot[car_dev_name]:get_equipped_offensive()

		armour:send_message( message.set_value, { value = armour_stat } )
		power:send_message( message.set_value, { value = power_stat } )
		offensive:send_message( message.set_value, { value = offensive_stat } )

		armour:show()
		power:show()
		offensive:show()
		armour_nav:show()
		power_nav:show()
		offensive_nav:show()
	else
		armour:hide()
		power:hide()
		offensive:hide()
		armour_nav:hide()
		power_nav:hide()
		offensive_nav:hide()
	end
end

on_push = function( menu )
	menu.bio_car = nil
	menu.equippedness = nil
	menu.ticker_message_bio = nil
	menu.ticker_message_equippedness = nil
end

--[[get_sorted_cars = function( menu )
	local sorted_cars = {}
	local num_cars = #menu.cars
	local max_tier = 1
	local car_counter = 1
	local cop_counter = 1
	local cops
	
	-- get max tier
	for i=1, num_cars do
		if menu.cars[i]:get_tier() > max_tier then
			max_tier = menu.cars[i]:get_tier()
		end
	end

	-- add the non cop cars without a tier
	for j=1, num_cars do
		local tier = menu.cars[j]:get_tier() 
		local owned = menu.cars[j]:is_owned()
		local cop = menu.cars[j]:is_cop()

		if tier == -1 and owned then
			if cop then
				if not cops then
					cops = {}
				end
				cops[cop_counter] = menu.cars[j]
				cop_counter = cop_counter + 1
			else
				sorted_cars[car_counter] = menu.cars[j]
				car_counter = car_counter + 1
			end
		end
	end
	
	-- for each tier add owned cars
	for i=1, max_tier do
		for j=1, num_cars do
			if menu.cars[j]:get_tier() == i then
				local owned = menu.cars[j]:is_owned()
				
				if owned then
					sorted_cars[car_counter] = menu.cars[j]
					car_counter = car_counter + 1
				end
			end
		end
	end
	
	-- for each tier add the non owned cars
	for i=1, max_tier do
		for j=1, num_cars do
			if menu.cars[j]:get_tier() == i then
				local owned = menu.cars[j]:is_owned()
				
				if not owned then
					sorted_cars[car_counter] = menu.cars[j]
					car_counter = car_counter + 1
				end
			end
		end
	end

	-- add the cops
	if cops then
		for i=1, #cops do
			sorted_cars[car_counter] = cops[i]
			car_counter = car_counter + 1
		end
	end
	
	return sorted_cars
end]]

on_refresh = function( menu )
	do_setup(menu)
end

do_setup = function( menu )
	-- fill spinner
	local car_dev_names = {}
	local car_names = {}
	if info_table.allowed_cars then
		menu.cars = info_table.allowed_cars
	elseif info_table.multiplayer then
		menu.cars = car_lot:get_usable_cars("mp")
	else -- if info_table.career then; -- also default case, even though this shouldn't happen
		menu.cars = car_lot:get_all_cars()
	end
	menu.cars = car_lot:get_all_cars()
	-- if not ( properties.menu_return ~= "online" or properties.menu_return ~= "lan" ) then
		-- menu.cars = get_sorted_cars( menu )
	-- end
	
	info_table.using_car_name = info_table.using_car_name or car_lot:get_player_car( 1, info_table.multiplayer and 2 or 1 )
	
	car_lot:unlock_all_skins()
	car_lot:unlock_all_rims()
	car_lot.squad_car:Unlock()
	for i=1, #car_lot do
	
	  car_lot[i]:Unlock()
	end
	
	for i=1, #menu.cars do
		local car_dev_name = menu.cars[i]:get_dev_name()
		car_dev_names[i] = car_dev_name
		car_names[i] = menu.cars[i]:get_name() or ""
	end

	-- add to list cars missing from content sys PLAYGO

--[[	for i=1, #car_list.base_cars do
		local dev_name = car_list.base_cars[i]
		if car_lot[dev_name] == nil then
			local dev_name = car_list.base_cars[i]
			local name = txt["FE_VEHICLE_"..string.upper(dev_name).."_UCASE"]

			table.insert(car_dev_names, dev_name)
			table.insert(car_names, name)
		end
	end]]

	if car_lot[info_table.using_car_name] == nil then -- car not present fallback to default PLAYGO
		info_table.using_car_name = "eagle_r"
	end


	selected_car_name = info_table.using_car_name
	menu:send_item_message( id.spinner, message.set_value, { text="", value=info_table.using_car_name, values=car_dev_names, lookup=car_names } )
	
	car_lot:set_garage_car( selected_car_name )

	--fill apo
	update_apo( menu )
	
	-- get the player's chosen skin/wheel for the car
	local skin_name = car_lot[selected_car_name]:get_player_skin()
	local wheel_name = car_lot[selected_car_name]:get_player_wheel()
	
	menu.model1 = menu:get_item_by_id( id.model1 )
	menu.model2 = menu:get_item_by_id( id.model2 )
	menu.focus_model = menu.model1
	menu.focus_model:set_alpha( 255 )
	menu.sw, menu.sh = ui:get_authored_size()
	menu.model1:send_message( message.set_value, { car_name=info_table.using_car_name, skin_name = skin_name, wheel_name = wheel_name } )
	menu.model2:set_position_x( menu.sw )
	menu.model2:set_alpha( 0 )
	
	full_w = menu.model1.w
	full_h = menu.model1.h
	scaled_w = info_scale*menu.model1.w
	scaled_h = info_scale*menu.model1.h
	
	render_state( menu, true )
	set_info( menu )
	set_button_bar( menu )
	
	set_lit( menu )
	
	change_state( menu, state or states.garage, true )
end

set_lit = function( menu )
	local lit = true
	if not info_table.multiplayer and  car_lot[selected_car_name] then
		--lit = car_lot[selected_car_name]:is_seen()
		--lit = car_lot[selected_car_name]:is_owned()
	end
	
	menu.focus_model:send_message( message.set_value, { lit = lit } )
end

on_focus = function( menu, is_focus )
	rich_presence:set_next_presence(2)
	if state == states.parts_shop and is_focus then
		setup_apo( menu )
	end
	
	if is_focus and state ~= states.parts_shop then 
		menu:refresh()
	end
end

on_pop = function( menu )
	set_ticker_bio( menu, nil ) -- releases ticker message
	set_ticker_equippedness( menu, nil )
	info_table = {}
	garage_cars:unload_vehicles()
	set_part_ticker_text( menu, false )
	-- set_rim_ticker_text( menu, false )
	-- set_skin_ticker_text( menu, false )
	
	if skin_unlock_msg then
		ticker_manager:release_message( skin_unlock_msg )
	end
	if rim_unlock_msg then
		ticker_manager:release_message( rim_unlock_msg )
	end
	if part_ticker_token then
		ticker_manager:release_message( part_ticker_token )
		part_ticker_token = nil
	end
end

on_update = function( menu )
	menu.focus_model:send_message( message.check_async, {} )
end

on_message = function( menu, msg_id, parms )
	if msg_id == message.set_value then
		if parms.state then
			state = states[parms.state]
		end
		
		if state == states.skins then
			original_slot_car = info_table.using_car_name
			if info_table.slot then
				original_slot_type = info_table.slot.racer.rtype
			end
		end
		
		pop_return = parms.pop_return
		customise_return = parms.customise_return
		
		if parms.info then
			info_table = parms.info
		end
	elseif msg_id == message.update then
		if parms.car then
			if info_table.using_car_name ~= parms.car then
				ui:pop_to( frontend.network_lobby )
				dlg_ok:send_message ( message.set_value, { title=txt.ui_fe_host_car_change_title_ucase, body=txt.ui_fe_host_car_change_body_ucase, } )
				ui:push( dlg_ok )
			end
		end
	end
end

----------------------------------------------------------------------------------------------------
-- customise
----------------------------------------------------------------------------------------------------

on_event[id.parts_shop] = function ( menu, item, evt )
	if evt == event.click then
		change_state( menu, states.parts_shop )
	end
end

on_event[id.skins] = function ( menu, item, evt )
	if evt == event.click then
		change_state( menu, states.skins )
	end
end

on_event[id.wheels] = function ( menu, item, evt )
	if evt == event.click then
		change_state( menu, states.wheels )
	end
end

----------------------------------------------------------------------------------------------------
-- parts shop
----------------------------------------------------------------------------------------------------

local dlg_yes_no 	= require "dialogue_boxes.yes_no"
local dlg_ok 		= require "dialogue_boxes.ok"
local dlg_ok_token			= require "dialogue_boxes.ok_token"
local dlg_yes_no_buy_part	= require "dialogue_boxes.yes_no_buy_part"

local first_index
local last_index

local car_name
local skin_name

local attribute_ids = { armour = 0, power = 1, offensive = 2 }
local attribute_names = { txt.ui_fe_apo_armour_ucase, txt.ui_fe_apo_power_ucase, txt.ui_fe_apo_offensive_ucase }

local owned_levels = {}
local equipped_levels = {}


local selected_cost
local selected_level
local selected_attribute

local part_ticker_token
local part_ticker_text

-- selected_item == false will clear the ticker
set_part_ticker_text = function( menu, selected_item )
	local ticker_text
	local icon = 1
	if selected_item == nil then
		selected_item = menu:get_selected_item()
	end
	if selected_item then
		local status = selected_item:send_message( message.get_value, { status = true } )
		if status == "locked" then
			ticker_text = txt.ui_fe_locked_apo_part_ticker
			icon = 2
		-- elseif status == "blocked" then
			-- ticker_text = txt.ui_fe_blocked_apo_part_ticker
		else
			ticker_text = selected_item:send_message( message.get_value, { ticker_text = true } )
		end
	end
	
	if ticker_text ~= part_ticker_text then
		if part_ticker_token then
			ticker_manager:release_message( part_ticker_token )
			part_ticker_token = nil
		end
		part_ticker_text = ticker_text
		if part_ticker_text then
			part_ticker_token = ticker_manager:add_messages( true, true, part_ticker_text, icon )
		end
	end
end

local previous_apo_button_id

button_event = function( menu, item, evt )
	if evt == event.click then
		local status, attribute, level = item:send_message( message.get_value, { status = true, attribute = true, level = true } )

		buy_name = car_name
		buy_group = attribute_ids[attribute]
		selected_level = level
		
		menu:send_item_message( id.apo_selector_container, message.set_value, { selected_level = level, } )
		
		selected_cost = menu:send_item_message( id.apo_selector_container, message.get_value, { cost = true } ) 
		selected_attribute = attribute
		
		if status == "owned" then
			if equip_allowed() then		
				if equipped_levels[selected_attribute] < selected_level then
					sound:play( sounds.remove_part ) 
				else
					sound:play( sounds.equip_part ) 
				end

				if selected_attribute == "armour" then
					car_lot[car_name]:set_equipped_armour( selected_level )
				elseif selected_attribute == "power" then
					car_lot[car_name]:set_equipped_power( selected_level )
				elseif selected_attribute == "offensive" then
					car_lot[car_name]:set_equipped_offensive( selected_level )
				end
				
				equipped_levels[selected_attribute] = selected_level
				menu:send_item_message( id.apo_selector_container, message.set_value, { equipped_levels = equipped_levels } )
				update_buttons( menu )
				update_equipped( menu )
				update_num_parts( menu )
				save_manager:save()
			else
				sound:play( sounds.impossible_action ) 
				local att_num = attribute_ids[selected_attribute]
				local counter = 1
				local t = {}
				for i=1, 3 do
					if i ~= att_num + 1 then
						t[counter] = attribute_names[i]
						counter = counter + 1
					end
				end
				local text = txt.ui_fe_apo_equip_fail_body_ucase
				text = text:gsub( "%0", t[1] )
				text = text:gsub( "%1", t[2] )
				
				dlg_ok:send_message ( message.set_value, { title=txt.ui_fe_apo_equip_fail_title_ucase, body=text, } )
				ui:push( dlg_ok )
			end

		elseif status == "available" then
			local affordable = menu:send_item_message( id.apo_selector_container, message.get_value, { affordable = true } )
			
			if affordable then
				local cost = menu:send_item_message( id.apo_selector_container, message.get_value, { cost = true } )
				
				local att_num = attribute_ids[selected_attribute]
				local att_text = attribute_names[att_num + 1]

				if equip_allowed() then
					local buy_text = txt.ui_fe_apo_buy_body_ucase

					local cost_text = txt.ui_fe_token_cost_ucase
					cost_text = cost_text:gsub( "%0", cost )

					local balance_text = txt.ui_fe_balance_ucase
					balance_text = balance_text:gsub( "%0", properties.apo_tokens )

					dlg_yes_no_buy_part:send_message ( message.set_value, { title=txt.ui_fe_apo_buy_title_ucase, body=buy_text, cost=cost_text, balance=balance_text, yes_func = apo_buy_yes, sound_activate = sounds.equip_part } )
					ui:push( dlg_yes_no_buy_part )
				else	
					local buy_text = txt.ui_fe_apo_buy_at_max_body_ucase
					buy_text = buy_text:gsub( "%0", att_text )
					buy_text = buy_text:gsub( "%1", selected_level )
					
					dlg_yes_no:send_message ( message.set_value, { title=txt.ui_fe_apo_buy_title_ucase, body=buy_text, yes_func = apo_buy_at_max_yes, sound_activate = sounds.button_activate } )
					ui:push( dlg_yes_no )
				end
			else
				sound:play( sounds.impossible_action ) 
				dlg_ok_token:send_message( message.set_value, { title=txt.ui_fe_apo_buy_fail_title_ucase, body=txt.ui_fe_apo_buy_fail_body_ucase, guy=false } )
				ui:push( dlg_ok_token )
			end
		elseif status == "locked" then
			sound:play( sounds.impossible_action ) 
			dlg_ok:send_message( message.set_value, { title=txt.ui_fe_apo_locked_title_ucase, body=txt.ui_fe_apo_locked_body_ucase } )
			ui:push( dlg_ok )
		end
		
	elseif evt == event.select then
		previous_apo_button_id = item:get_item_id()
		local level = item:send_message( message.get_value, { level = true } )
		menu:send_item_message( id.apo_selector_container, message.set_value, { selected_level = level, owned_levels = owned_levels, equipped_levels = equipped_levels, balance = properties.apo_tokens } )
		set_part_ticker_text( menu )
	elseif evt == event.highlight then
		local level = item:send_message( message.get_value, { level = true } )
		menu:send_item_message( id.apo_selector_container, message.set_value, { selected_level = level, owned_levels = owned_levels, equipped_levels = equipped_levels, balance = properties.apo_tokens } )
		set_part_ticker_text( menu )
	elseif evt == event.dehighlight then
		if previous_apo_button_id then
			local prev_item = menu:get_item_by_id( previous_apo_button_id )
			local level = prev_item:send_message( message.get_value, { level = true } )
			menu:send_item_message( id.apo_selector_container, message.set_value, { selected_level = level, owned_levels = owned_levels, equipped_levels = equipped_levels, balance = properties.apo_tokens } )
			set_part_ticker_text( menu )
		end
	end
end

on_event[id.armour1] = button_event
on_event[id.armour2] = button_event
on_event[id.armour3] = button_event
on_event[id.armour4] = button_event
on_event[id.armour5] = button_event

on_event[id.power1] = button_event
on_event[id.power2] = button_event
on_event[id.power3] = button_event
on_event[id.power4] = button_event
on_event[id.power5] = button_event

on_event[id.offensive1] = button_event
on_event[id.offensive2] = button_event
on_event[id.offensive3] = button_event
on_event[id.offensive4] = button_event
on_event[id.offensive5] = button_event

on_event[id.armour_nav] = function( menu, item, evt )
	if state == states.parts_shop then
		if evt == event.click then
			local attribute = menu:send_item_message( id.apo_selector_container, message.get_value, { attribute = true } )
			menu:send_item_message( id.apo_selector_container, message.move, { index = 1 } )
			
			if attribute == "power" then
				change_attribute ( menu, -1 )
			elseif attribute == "offensive" then
				change_attribute ( menu, -2 )
			end
		end
	end
end

on_event[id.power_nav] = function( menu, item, evt )
	if state == states.parts_shop then
		if evt == event.click then
			local attribute = menu:send_item_message( id.apo_selector_container, message.get_value, { attribute = true } )
			menu:send_item_message( id.apo_selector_container, message.move, { index = 2 } )
			
			if attribute == "armour" then
				change_attribute ( menu, 1 )
			elseif attribute == "offensive" then
				change_attribute ( menu, -1 )
			end
		end
	end
end

on_event[id.offensive_nav] = function( menu, item, evt )
	if state == states.parts_shop then
		if evt == event.click then
			local attribute = menu:send_item_message( id.apo_selector_container, message.get_value, { attribute = true } )
			menu:send_item_message( id.apo_selector_container, message.move, { index = 3 } )
			
			if attribute == "armour" then
				change_attribute ( menu, 2 )
			elseif attribute == "power" then
				change_attribute ( menu, 1 )
			end
		end
	end
end

on_event[id.apo_selector_container] = function( menu, item, evt )
	if state == states.parts_shop then
		if evt == event.moved_left then
			change_attribute( menu, -1 )
		elseif evt == event.moved_right then
			change_attribute( menu, 1 )
		end
	end
end

on_input = function( menu, input )
	if state == states.parts_shop then
		if input:control_repeating( CONTROL_FRONTEND_LEFT ) then
			menu:send_item_message( id.apo_selector_container, message.move, { left = true, } )
			change_attribute( menu, -1 )
			return true
		elseif input:control_repeating( CONTROL_FRONTEND_RIGHT ) then
			menu:send_item_message( id.apo_selector_container, message.move, { right = true, } )
			change_attribute( menu, 1 )
			return true
		end
		return false
	end
end

get_car_name = function()
	local car_name = car_lot:get_garage_car()
	
	if not car_name or car_name == "" then
		car_name = car_lot:get_player_car( 1, 1 )
	end
	
	return car_name
end

setup_apo = function( menu )
	if not first_index then
		local first_id = garage_common.get_first_id()
		local item = menu:get_item_by_id( first_id )
		first_index = item:get_index()
	end
	if not last_index then
		local last_id = garage_common.get_last_id()
		local item = menu:get_item_by_id( last_id )
		last_index = item:get_index()
	end
	
	local background = menu:get_item_by_id( id.event_background )
	background:set_position_x( event_selector_intro_x )
	background:move( -event_selector_intro_x, 0, event_selector_bkg_intro_time, transition.linear )
	
	car_name = get_car_name()
	
	owned_levels["armour"] = car_lot[car_name]:get_owned_armour()
	owned_levels["power"] = car_lot[car_name]:get_owned_power()
	owned_levels["offensive"] = car_lot[car_name]:get_owned_offensive()
	
	equipped_levels["armour"] = car_lot[car_name]:get_equipped_armour()
	equipped_levels["power"] = car_lot[car_name]:get_equipped_power()
	equipped_levels["offensive"] = car_lot[car_name]:get_equipped_offensive()
	
	update_equipped( menu )
	
	local armour_data_table = ApoManager:get_apo_data_table( car_name, 0 )
	local power_data_table = ApoManager:get_apo_data_table( car_name, 1 )
	local offensive_data_table = ApoManager:get_apo_data_table( car_name, 2 )
	
	local career = require ( "career."..properties.career_current )
	local data = career.playlists
	
	garage_common.set_unlock_levels( data )
	
	initialise_buttons( menu, "armour", armour_data_table )
	initialise_buttons( menu, "power", power_data_table )
	initialise_buttons( menu, "offensive", offensive_data_table )

	menu:send_item_message( id.apo_selector_container, message.set_value, { 
		initialise = true,
		data = data,
		armour = armour_data_table, 
		power = power_data_table, 
		offensive = offensive_data_table, 
		owned_levels = owned_levels,
		equipped_levels = equipped_levels,
		unlock_levels = garage_common.get_unlock_levels(),
		balance = properties.apo_tokens } )
	
	update_parts_menu( menu )

	update_num_parts( menu )
	
	-- local text = txt.fe_token_count_ucase
	-- text = text:gsub( "%0", properties.apo_tokens )
	menu:send_item_message( id.apo_tokens, message.set_text, { text = properties.apo_tokens } )
end

update_equipped = function( menu )
	local armour = menu:get_item_by_id( id.armour )
	local power = menu:get_item_by_id( id.power )
	local offensive = menu:get_item_by_id( id.offensive )

	armour:send_message( message.set_value, { value = equipped_levels["armour"] } )
	power:send_message( message.set_value, { value = equipped_levels["power"] } )
	offensive:send_message( message.set_value, { value = equipped_levels["offensive"] } )
end

initialise_buttons = function( menu, attribute, data )	
	for i=1, #data do	
		local button = menu:get_item_by_id( id[attribute..i] )
		local status
		local sound_activate
		
		if i == equipped_levels[attribute] then
			status = "equipped"
			sound_activate = sounds.impossible_action
		elseif 0 <= owned_levels[attribute] then
			status = "owned"
			sound_activate = sounds.equip_part
		else			
			local unlock_level = garage_common.get_unlock_level( attribute )
			
			if 0 <= unlock_level then
				status = "available"
				sound_activate = sounds.button_activate
			else
				status = "locked"
				sound_activate = sounds.impossible_action
			end
		end
		status = "owned"
		button:send_message( message.set_value, { text = txt[data[i].name], status = status, sound_activate = sound_activate } )
	end
end

update_parts_menu = function( menu )
	local attribute = menu:send_item_message( id.apo_selector_container, message.get_value, { attribute = true } )
	
	set_button_visibility( menu, attribute )
	set_navigation_buttons( menu, attribute )
	set_info_text( menu, attribute )
end

set_info_text = function( menu, attribute )
	-- local text = "ui_fe_apo_"..attribute.."_ucase"
	-- menu:send_item_message( id.apo_selector_container, message.set_text, { text = txt[text] } )
	menu:send_item_message( id.apo_selector_container, message.update_button_bar )
end

set_navigation_buttons = function( menu, attribute )
	if attribute == "armour" then
		menu:send_item_message( id.armour_nav, message.play, { animation = "active" } )
		menu:send_item_message( id.power_nav, message.play, { animation = "default" } )
		menu:send_item_message( id.offensive_nav, message.play, { animation = "default" } )
	elseif attribute == "power" then
		menu:send_item_message( id.armour_nav, message.play, { animation = "default" } )
		menu:send_item_message( id.power_nav, message.play, { animation = "active" } )
		menu:send_item_message( id.offensive_nav, message.play, { animation = "default" } )
	else		
		menu:send_item_message( id.armour_nav, message.play, { animation = "default" } )
		menu:send_item_message( id.power_nav, message.play, { animation = "default" } )
		menu:send_item_message( id.offensive_nav, message.play, { animation = "active" } )
	end
end

set_button_visibility = function( menu, attribute )
	if attribute == "armour" then
		menu:show_group( id.armour )
		menu:hide_group( id.power )
		menu:hide_group( id.offensive )
	elseif attribute == "power" then
		menu:hide_group( id.armour )
		menu:show_group( id.power )
		menu:hide_group( id.offensive )
	else		
		menu:hide_group( id.armour )
		menu:hide_group( id.power )
		menu:show_group( id.offensive )
	end
end

get_remaining_levels_to_equip = function()
	local diff_from_equipped = selected_level - equipped_levels[selected_attribute]
	local car_name = get_car_name()
	local new_parts_equipped = garage_common.get_parts_in_use( car_name ) + diff_from_equipped
	return new_parts_equipped - garage_common.get_max_parts()
end

equip_allowed = function()
	local diff_from_equipped = selected_level - equipped_levels[selected_attribute]
	local car_name = get_car_name()
	local new_parts_equipped = garage_common.get_parts_in_use( car_name ) + diff_from_equipped
	local allowed = ( new_parts_equipped <= garage_common.get_max_parts() )
	return allowed
end

update_buttons = function( menu )
	local data_table = ApoManager:get_apo_data_table( car_name, attribute_ids[selected_attribute] )
	initialise_buttons( menu, selected_attribute, data_table )
end

update_num_parts = function( menu )
	local car_name = get_car_name()
	-- local parts_remaining = garage_common.get_max_parts() - garage_common.get_parts_in_use( car_name )
	-- local text = txt.fe_apo_parts_left_ucase
	--text = text:gsub( "%0", parts_remaining )
	--menu:send_item_message( id.apo_remaining, message.set_value, { parts_remaining = parts_remaining, text = text,  } )
	
	local parts_equipped = txt.ui_fe_parts_equipped_ucase
	
	local equipped = garage_common.get_parts_in_use( car_name )
	local max_parts = garage_common.get_max_parts()
	local values = txt.ui_fe_parts_equipped_values
	values = values:gsub( "%0", equipped )
	values = values:gsub( "%1", max_parts )
	
	menu:send_item_message( id.apo_remaining, message.set_text, {  text = parts_equipped, body_text = values } )
end

change_attribute = function( menu, dir )
	menu:show_group( id.armour )
	menu:show_group( id.power )
	menu:show_group( id.offensive )

	local button = menu:get_selected_item()
	local button_index = button:get_index() 
	local num_garage_levels = garage_common.get_num_levels()
	local new_index = button_index + ( dir * num_garage_levels )	
	
	--[[if new_index < first_index then
		local diff = new_index - first_index
		new_index = last_index + diff + 1
	elseif new_index > last_index then
		local diff = new_index - last_index
		new_index = first_index + diff - 1
	end]]
	
	if new_index >= first_index and new_index <= last_index then
		menu:select_item_by_index( new_index )
	end
	
	local attribute = menu:send_item_message( id.apo_selector_container, message.get_value, { attribute = true } )
	
	set_button_visibility( menu, attribute )
	set_navigation_buttons( menu, attribute )
	set_info_text( menu, attribute )
end

-- apo_sell_yes = function()
	-- if selected_attribute == "armour" then
		-- car_lot[car_name]:set_equipped_armour( selected_level )
	-- elseif selected_attribute == "power" then
		-- car_lot[car_name]:set_equipped_power( selected_level )
	-- elseif selected_attribute == "offensive" then
		-- car_lot[car_name]:set_equipped_offensive( selected_level )
	-- end
	
	-- equipped_levels[selected_attribute] = selected_level
	
	-- save_manager:save()
	
	--[[stats:add_stat("number_of_bought_parts", 1)
	if get_remaining_levels_to_equip() == 0 then
		-- we have bought and equipped a part that is the last we could equip
		-- it means we have completed this car
		stats:add_stat("fully_upgraded_cars", 1)
	end]]
	
	-- ui:pop()
-- end

apo_buy_yes = function()
	-- buy it and equip it
	properties.apo_tokens = properties.apo_tokens - selected_cost
	
	if selected_attribute == "armour" then
		car_lot[car_name]:set_owned_armour( selected_level )
		car_lot[car_name]:set_equipped_armour( selected_level )
	elseif selected_attribute == "power" then
		car_lot[car_name]:set_owned_power( selected_level )
		car_lot[car_name]:set_equipped_power( selected_level )
	elseif selected_attribute == "offensive" then
		car_lot[car_name]:set_owned_offensive( selected_level )
		car_lot[car_name]:set_equipped_offensive( selected_level )
	end
	
	owned_levels[selected_attribute] = selected_level
	equipped_levels[selected_attribute] = selected_level
	
	stats:add_stat("number_of_bought_parts", 1)
	if(userstats~=nil) then
		userstats:send_player_part_unlocked(car_lot[car_name]:get_leaderboard_id())
	end
	if get_remaining_levels_to_equip() >= 0 then
		-- we have bought and equipped a part that is the last we could equip
		-- it means we have completed this car
		stats:add_stat("fully_upgraded_cars", 1)
		if(userstats~=nil) then
			userstats:send_player_fully_upgraded_car(car_lot[car_name]:get_leaderboard_id())
		end
	end
	
	save_manager:save()
	ui:pop()
end

apo_buy_at_max_yes = function()
	-- buy it
	properties.apo_tokens = properties.apo_tokens - selected_cost
	
	if selected_attribute == "armour" then
		car_lot[car_name]:set_owned_armour( selected_level )
	elseif selected_attribute == "power" then
		car_lot[car_name]:set_owned_power( selected_level )
	elseif selected_attribute == "offensive" then
		car_lot[car_name]:set_owned_offensive( selected_level )
	end
	
	owned_levels[selected_attribute] = selected_level
	
	stats:add_stat("number_of_bought_parts", 1)
	if(userstats~=nil) then
		userstats:send_player_part_unlocked(car_lot[car_name]:get_leaderboard_id())
	end
	if get_remaining_levels_to_equip() >= 0 then
		-- we have bought and equipped a part that is the last we could equip
		-- it means we have completed this car
		stats:add_stat("fully_upgraded_cars", 1)
		if(userstats~=nil) then
			userstats:send_player_fully_upgraded_car(car_lot[car_name]:get_leaderboard_id())
		end
	end
	
	save_manager:save()
	ui:pop()
end

----------------------------------------------------------------------------------------------------
-- skins
----------------------------------------------------------------------------------------------------

-- if the following variables change, pop the menu
local original_slot_type
local original_slot_car

-- local skin_ticker_token
-- local skin_ticker_text

-- selected_item == false will clear the ticker
-- set_skin_ticker_text = function( menu, selected_item )
	-- local ticker_text
	-- if selected_item then
		-- ticker_text = selected_item:send_message( message.get_value, { ticker_text = true } )
	-- end
	
	-- if ticker_text ~= skin_ticker_text then
		-- if skin_ticker_token then
			-- ticker_manager:release_message( skin_ticker_token )
			-- skin_ticker_token = nil
		-- end
		-- skin_ticker_text = ticker_text
		-- if skin_ticker_text then
			-- skin_ticker_token = ticker_manager:add_messages( true, true, skin_ticker_text, 1 )
		-- end
	-- end
-- end

check_skin = function( menu )
	if state == states.skins then
		-- if last_skin_selection then
			-- local locked = last_skin_selection:send_message( message.get_value, { locked = true } )
			
			-- if locked and prev_unlocked_skin then
				-- local equipped_wheel = car_lot[selected_car_name]:get_player_wheel()
				-- menu.focus_model:send_message( message.set_value, { paint = true, car_name = selected_car_name, skin_name = prev_unlocked_skin, wheel_name = equipped_wheel } )
			-- end
		-- end
		
		local equipped_skin = car_lot[selected_car_name]:get_player_skin()

		local selection = menu:send_item_message( id.skin_list, message.get_selection )
		local skin_name = selection:send_message( message.get_value )

		if skin_name ~= equipped_skin then
			local equipped_wheel = car_lot[selected_car_name]:get_player_wheel()
			menu.focus_model:send_message( message.set_value, { paint = true, car_name = selected_car_name, skin_name = equipped_skin, wheel_name = equipped_wheel } )
		end
	end
end

on_event[id.skin_list] = function( menu, item, evt )
	if evt == event.value_click then
		local selection = menu:send_item_message( id.skin_list, message.get_selection )
		local locked = false -- selection:send_message( message.get_value, { locked = true } )
		
		if selection then	
			last_skin_selection = selection
			local sub_menu = menu:send_item_message( id.skin_list, message.get_content, {} )
	
			if sub_menu and prev_skin_id and not locked then
				sub_menu:send_item_message( prev_skin_id, message.hide_equipped )
				prev_skin_id = selection:get_item_id()
				sub_menu:send_item_message( prev_skin_id, message.show_equipped )
			end

			local selected_skin = selection:send_message( message.get_value )
			
			local car_name = get_car_name()
			local equipped_skin = car_lot[selected_car_name]:get_player_skin()
			local equipped_wheel = car_lot[selected_car_name]:get_player_wheel()
		
			--if equipped_skin ~= selected_skin then
				if not locked then
					prev_unlocked_skin = selected_skin
					-- info_table.using_car_name = car_name
					local set_car = ( info_table.using_car_name == car_name )
					car_lot:set_player_car( 1, info_table.multiplayer and 2 or 1, car_name, selected_skin, equipped_wheel, set_car )
					
					if info_table.lobby_menu and car_name == info_table.using_car_name then
						info_table.lobby_menu:send_message( message.car_selected, {on_grid = false, car = selected_car_name, skin = selected_skin, wheel=equipped_wheel, set_car = set_car, slot=info_table.slot} )
					end
					
					check_wheel( menu )
				end
				
				set_button_bar( menu )
			--end
		end
	elseif evt == event.value_change then
		local selection = item:send_message( message.get_selection )
		local locked = false -- selection:send_message( message.get_value, { locked = true } )
		local car_name = get_car_name()
		local skin_name = selection:send_message( message.get_value )
		--set_skin_ticker_text( menu, selection )

		local challenge = stats:get_challenge_using_skin( car_name, skin_name )
		local text

		local prev_skin_type = skin_type
		
		if skin_name == "default" then
			skin_type = "default"
			text = txt.fe_car_skin_desc_default_ucase
		elseif challenge and challenge ~= "" then
			if locked then
				if prev_skin_type == "locked" then
					skin_type = "other_locked"
				else
					skin_type = "locked"
				end
				
				local challenge_name = txt["fe_challenge_"..challenge.."_name"]
				text = txt.fe_challenge_skin_locked_ucase
				text = text:gsub( "%0", challenge_name )
			else
				skin_type = "mp"
				text = txt.fe_car_skin_desc_mp_ucase
			end
		elseif car_name == "eagle_r" and skin_name == "paint_09" then
			skin_type = "kickstarter"
			text = txt.fe_car_skin_desc_kickstarter_ucase
		else
			skin_type = "mp"
			text = txt.fe_car_skin_desc_mp_ucase
		end
		
		if skin_type ~= prev_skin_type then
			if skin_unlock_msg then
				ticker_manager:release_message( skin_unlock_msg )
			end
			
			skin_unlock_msg = ticker_manager:add_messages( true, true, text, 1 )
		end
		
		local equipped_wheel = car_lot[selected_car_name]:get_player_wheel()
		menu.focus_model:send_message( message.set_value, { paint = true, car_name = selected_car_name, skin_name = skin_name, wheel_name = equipped_wheel } )
		
		set_button_bar( menu )
	elseif evt == event.highlight then
		-- highlighted = item:send_message( message.get_highlight )
		set_button_bar( menu )
	elseif evt == event.dehighlight then
		-- selection = item:send_message( message.get_selection )
		set_button_bar( menu )
	end
end

populate_skins = function( menu )
	local items = {}
	local car_name = get_car_name()
	local player_skin = car_lot[car_name]:get_player_skin()
	
	local unlocked_skins = car_lot[car_name]:get_unlocked_skins()
	
	for i,skin in ipairs( unlocked_skins ) do
		local dev_name = skin:get_dev_name()
		local name = skin:get_name()
		local item_id = "skin"..(#items+1)
		items[#items+1] = button_value_small_paged{ id=id[item_id], equipable = true, value=dev_name, text=name, sound_activate = sounds.change_skin, ticker_text=skin:get_ticker_text() }
		if dev_name == player_skin then
			prev_skin_id = id[item_id]
		end
	end
	
	local locked_skins = car_lot[car_name]:get_locked_skins()
	
	for i,skin in ipairs( locked_skins ) do
		local dev_name = skin:get_dev_name()
		local name = skin:get_name()
		local item_id = "skin"..(#items+1)
		items[#items+1] = button_value_small_paged{ locked=false, id=id[item_id], equipable = true, value=dev_name, text=name, sound_activate = sounds.impossible_action, ticker_text=skin:get_ticker_text() }
		if dev_name == player_skin then
			prev_skin_id = id[item_id]
		end
	end
		
	menu:send_item_message( id.skin_list, message.set_new_content, { content = items, stripey = true } )
	
	local sub_menu = menu:send_item_message( id.skin_list, message.get_content, {} )
	
	if sub_menu and prev_skin_id then
		sub_menu:send_item_message( prev_skin_id, message.show_equipped )
	end
	
	menu:send_item_message( id.skin_list, message.set_selection, { item_id = prev_skin_id } )
	menu:send_item_message( id.skin_list, message.move, { into_view = true } )
	menu:check_focus()
	
	return #items
end

check_slot_status = function( menu )
	if state == states.skins then
		if original_slot_type ~= info_table.slot.racer.rtype or original_slot_car ~= info_table.slot:send_message(message.get_value, { car=true } ) then
			if pop_return then
				ui:pop()
			else
				change_state( menu, states.customise )
			end
		end	
	end
end

-- function InNetworkSession(menu)
	-- if info_table.lobby_menu == frontend.network_lobby and not properties.in_network_session then
		-- ui:pop()
	-- end
-- end
-- function PlayerListChange(menu)
	-- if not properties.in_network_session then return end
	-- if info_table.lobby_menu and info_table.lobby_menu == frontend.network_lobby then
		-- info_table.lobby_menu:send_message( message.set_value, {PlayerListChange=true} )
		-- check_slot_status( menu )
	-- end
-- end
-- function PlayerChange(menu)
	-- if not properties.in_network_session then return end
	-- if info_table.lobby_menu and info_table.lobby_menu == frontend.network_lobby then
		-- info_table.lobby_menu:send_message( message.set_value, {PlayerChange=true} )
		-- check_slot_status( menu )
	-- end
-- end
-- function finallyStarting(menu)
	-- if info_table.lobby_menu and info_table.lobby_menu == frontend.network_lobby and NetworkGame.start_loading then
		-- ui:pop()
	-- end
-- end

-- add_item( watch{ handler = InNetworkSession, 		property_watch="in_network_session" 	} )
-- add_item( watch{ handler = PlayerListChange, 		property_watch="player_refresh" 	} )
-- add_item( watch{ handler = PlayerChange, 			property_watch="player_property_changed" 	} )
-- add_item( watch{ handler = finallyStarting,			network_property_watch="start_loading" } )

----------------------------------------------------------------------------------------------------
-- wheels
----------------------------------------------------------------------------------------------------

-- local rim_ticker_token
-- local rim_ticker_text

-- selected_item == false will clear the ticker
-- set_rim_ticker_text = function( menu, selected_item )
	-- local ticker_text
	-- if selected_item then
		-- ticker_text = selected_item:send_message( message.get_value, { ticker_text = true } )
	-- end
	
	-- if ticker_text ~= rim_ticker_text then
		-- if rim_ticker_token then
			-- ticker_manager:release_message( rim_ticker_token )
			-- rim_ticker_token = nil
		-- end
		-- rim_ticker_text = ticker_text
		-- if rim_ticker_text then
			-- rim_ticker_token = ticker_manager:add_messages( false, false, rim_ticker_text, 1 )
		-- end
	-- end
-- end

check_rim = function( menu )
	if state == states.wheels then
		-- if last_rim_selection then
			-- local locked = last_rim_selection:send_message( message.get_value, { locked = true } )
			
			-- if locked and prev_unlocked_rim then
				-- local equipped_skin = car_lot[selected_car_name]:get_player_skin()
				-- menu.focus_model:send_message( message.set_value, { wheels = true, car_name = selected_car_name, skin_name = equipped_skin, wheel_name = prev_unlocked_rim } )
			-- end
		-- end
		
		local equipped_wheel = car_lot[selected_car_name]:get_player_wheel()
		
		local selection = menu:send_item_message( id.wheel_list, message.get_selection )
		local wheel_name = selection:send_message( message.get_value )
		
		if wheel_name ~= equipped_wheel then
			local equipped_skin = car_lot[selected_car_name]:get_player_skin()
			menu.focus_model:send_message( message.set_value, { wheels = true, car_name = selected_car_name, skin_name = equipped_skin, wheel_name = equipped_wheel } )
		end
	end
end

on_event[id.wheel_list] = function( menu, item, evt )
	if evt == event.value_click then
		local selection = menu:send_item_message( id.wheel_list, message.get_selection )
		local locked = false -- selection:send_message( message.get_value, { locked = true } )
		
		if selection then
			last_rim_selection = selection
			local sub_menu = menu:send_item_message( id.wheel_list, message.get_content, {} )
	
			if sub_menu and prev_wheel_id and not locked then
				sub_menu:send_item_message( prev_wheel_id, message.hide_equipped )
				prev_wheel_id = selection:get_item_id()
				sub_menu:send_item_message( prev_wheel_id, message.show_equipped )
			end
		
			local selected_wheel = selection:send_message( message.get_value )
			
			local car_name = get_car_name()
			local equipped_skin = car_lot[selected_car_name]:get_player_skin()
			local equipped_wheel = car_lot[selected_car_name]:get_player_wheel()
		
			--if equipped_wheel ~= selected_wheel then
				if not locked then
					prev_unlocked_rim = selected_wheel
					-- info_table.using_car_name = car_name
					local set_car = ( info_table.using_car_name == car_name )
					car_lot:set_player_car( 1, info_table.multiplayer and 2 or 1, car_name, equipped_skin, selected_wheel, set_car )
					
					if info_table.lobby_menu then
						info_table.lobby_menu:send_message( message.car_selected, {on_grid = false, car = selected_car_name, skin = equipped_skin, wheel=selected_wheel, set_car = set_car, slot=info_table.slot} )
					end
					
					check_wheel( menu )
				end

				set_button_bar( menu )
			--end
		end
	elseif evt == event.value_change then
		local selection = item:send_message( message.get_selection )
		local locked = false --selection:send_message( message.get_value, { locked = true } )
		local car_name = get_car_name()
		local rim_name = selection:send_message( message.get_value )
		--set_rim_ticker_text( menu, selection )
		
		local challenge = stats:get_challenge_using_rim( car_name, rim_name )
		local text
		
		local prev_wheel_type = wheel_type
		
		if rim_name == "default" then
			wheel_type = "default"
			text = txt.fe_car_rim_desc_default_ucase
		elseif challenge and challenge ~= "" then
			if locked then
				if prev_wheel_type == "locked" then
					wheel_type = "other_locked"
				else
					wheel_type = "locked"
				end
				
				local challenge_name = txt["fe_challenge_"..challenge.."_name"]
				text = txt.fe_challenge_rim_locked_ucase
				text = text:gsub( "%0", challenge_name )
			else
				wheel_type = "alt"
				text = txt.fe_car_rim_desc_alternate_ucase
			end
		else
			wheel_type = "alt"
			text = txt.fe_car_rim_desc_alternate_ucase
		end
		
		if wheel_type ~= prev_wheel_type then
			if rim_unlock_msg then
				ticker_manager:release_message( rim_unlock_msg )
			end
			
			rim_unlock_msg = ticker_manager:add_messages( true, true, text, 1 )
		end
		
		local equipped_skin = car_lot[selected_car_name]:get_player_skin()
		menu.focus_model:send_message( message.set_value, { wheels = true, car_name = selected_car_name, skin_name = equipped_skin, wheel_name = rim_name } )
		
		set_button_bar( menu )
	elseif evt == event.highlight then
		-- highlighted = item:send_message( message.get_highlight )
		set_button_bar( menu )
	elseif evt == event.dehighlight then
		-- selection = item:send_message( message.get_selection )
		set_button_bar( menu )
	end
end

populate_wheels = function( menu )
	local items = {}
	local car_name = get_car_name()
	local player_wheels = car_lot[car_name]:get_player_wheel()
	
	local unlocked_wheels = car_lot[car_name]:get_unlocked_wheels()
	
	for i,wheel in ipairs( unlocked_wheels ) do
		local dev_name = wheel:get_group_name()
		local item_id = "wheel"..(#items+1)
		items[#items+1] = button_value_small_paged{ id=id[item_id], equipable = true, value=dev_name, text=wheel:get_name(), sound_activate = sounds.change_wheel, ticker_text=wheel:get_ticker_text() }
		if dev_name == player_wheels then
			prev_wheel_id = id[item_id]
		end
	end
	
	local locked_wheels = car_lot[car_name]:get_locked_wheels()
	
	for i,wheel in ipairs( locked_wheels ) do
		local dev_name = wheel:get_group_name()
		local item_id = "wheel"..(#items+1)
		items[#items+1] = button_value_small_paged{ locked=false, id=id[item_id], equipable = true, value=dev_name, text=wheel:get_name(), sound_activate = sounds.impossible_action, ticker_text=wheel:get_ticker_text() }
		if dev_name == player_wheels then
			prev_wheel_id = id[item_id]
		end
	end
		
	menu:send_item_message( id.wheel_list, message.set_new_content, { content = items, stripey = true } )
	
	local sub_menu = menu:send_item_message( id.wheel_list, message.get_content, {} )
	
	if sub_menu and prev_wheel_id then
		sub_menu:send_item_message( prev_wheel_id, message.show_equipped )
	end
	
	menu:send_item_message( id.wheel_list, message.set_selection, { item_id = prev_wheel_id } )
	menu:send_item_message( id.wheel_list, message.move, { into_view = true } )
	menu:check_focus()
	
	return #items
end

dlc_packages_available_changed = function(menu)
dprint("dlc_packages_available_changed called")
	local allowed_cars
    local car_selection
    local cars_available = properties.cars_available
    
    if info_table.multiplayer then
                    car_selection = properties.profile_car_selection_mp
                    
                    if car_selection == "chosen" or car_selection == "random" then
                                    if cars_available == "hosts_unlocked_cars" then
                                                    allowed_cars = car_lot:get_set_of_cars( {cop=false, progress=2, sort_by_tier=true } )
                                    elseif cars_available == "rounded" then
                                                    allowed_cars = car_lot:get_set_of_cars( {cop=false, progress=2, sort_by_tier=true, class_rounded=true, class_speedy=false, class_smashy=false } )
                                    elseif cars_available == "speedy" then
                                                    allowed_cars = car_lot:get_set_of_cars( {cop=false, progress=2, sort_by_tier=true, class_rounded=false, class_speedy=true, class_smashy=false } )
                                    elseif cars_available == "smashy" then
                                                    allowed_cars = car_lot:get_set_of_cars( {cop=false, progress=2, sort_by_tier=true, class_rounded=false, class_speedy=false, class_smashy=true } )
                                    elseif cars_available == "eagles_only" then
                                                    allowed_cars = car_lot:get_set_of_cars( {cop=false, progress=2, sort_by_tier=true, eagle=true } )
                                    elseif cars_available == "custom" then
                                                    local network_car_manager = require "frontend.network_car_manager"
                                                    allowed_cars = network_car_manager.get_custom_car_list()
                                    end
                    else
                                    -- car_selection == "hosts_choice"
                                    allowed_cars = car_lot:get_set_of_cars( {cop=false, progress=2, sort_by_tier=true } ) -- CLuaCarLotCar::Progress enum
                    end
    else
                    allowed_cars = car_lot:get_set_of_cars( { sort_by_tier=true, single_player=true } )
    end
    
    info_table.allowed_cars = allowed_cars
    do_setup(menu)
    menu.focus_model:send_message( message.set_value, { lit = true } )
end

add_item( watch{ handler = dlc_packages_available_changed, 	property_watch="dlc_packages_available" 	} )
