module( ..., lube.seeall)

local playlists = require "playlists"
local progress_manager = require "frontend.progress_manager"
local dlg_ok 			= require "dialogue_boxes.ok"

local setup_menu
local index = nil
local new_event = nil
local event_added
local route = nil
local select_change_ticker_delay = 500 -- in ms
local event_spamming_stopped = "event_spamming_stopped"
local last_lockedness = nil
local original_route = nil
local route_selected = nil
local random_route
local item_id
local last_item_id

change_ticker = function( menu )
	if item_id ~= last_item_id then
		ticker_manager:release_message( menu.ticker_message_desc )
		
		if route.locked then
			local text = txt.ui_fe_ticker_unlock_route_ucase
			text = text:gsub("%0", route.locked )
			menu.ticker_message_desc = ticker_manager:add_messages( true, true, text, 2 )
		-- else
			-- menu.ticker_message_desc = ticker_manager:add_messages( true, true, route.writeup, 4 )
		end
		
		last_item_id = item_id
	end
end

selection_change = function( menu )
	menu:get_item_by_id( id.timer ).timer:set( select_change_ticker_delay, event_spamming_stopped )
end

route_list =  function()
	return function( menu, item, evt )
		local selected_race = nil
		
		if evt == event.value_change then
			selected_race = item:send_message( message.get_selection )
		elseif evt == event.highlight then
			selected_race = item:send_message( message.get_highlight )
		elseif evt == event.dehighlight then
			selected_race = item:send_message( message.get_selection )
		end
		
		if selected_race then
			item_id = selected_race:get_item_id()
			
			route = selected_race:send_message( message.get_value, {} )

			if setup_menu then
				if not route.image or route.image == "" then
					route.image = true
				end
				
				if item_id == id.random then
					random_route = true
				else
					random_route = false
				end

				setup_menu:send_message( message.set_value, { change_image = true, race_thumbnail = route.image, level_name = route.name , download_in_progress =  route.download_in_progress } )
				selection_change( menu )
				
				local new_locked = route.locked
				local anim
				if last_lockedness == nil and new_locked then
					anim = "select_locked"
				elseif last_lockedness and new_locked == nil then
					anim = "deselect_locked"
				end
				last_lockedness = new_locked
				if anim then
					setup_menu:send_message( message.set_value, { play_anim=true, state=anim } )
				end
			end
		end
		
		if evt == event.value_click then
		
			local sel_route = item:send_message( message.get_selection )
			local value = sel_route:send_message( message.get_value )


			if value.download_in_progress then
				local progress = Application:StreamingDownloadProgress()
				local title = txt.ui_fe_install_dialog_title_ucase:gsub("%d", tostring(progress) ):gsub("%%", "%")
				local text = txt.ui_fe_install_remainder:gsub("%d", tostring(progress) ):gsub("%%", "%")
				dlg_ok:send_message ( message.set_value, { title=title, body=text, } )
				ui:push( dlg_ok )
			else
			
				if not value.locked then
					route_selected = true
					event_added = true
				
			--		playlists.current[index].name 		= value.name
			--		playlists.current[index].image 		= value.image
					playlists.current[index].environment 	= value.environment
					playlists.current[index].level 		= value.level
					playlists.current[index].race 		= value.race
					
					ui:pop_to( setup_menu )
				else
					setup_menu:send_message( message.set_value, { play_anim=true, state="activate_locked" } )
				end
			end
		end
		
		--if evt == event.value_click then	
		--	ui:push( require "frontend.paged_list_test" )
		--end
	end
end

pg_up_button = function()
	return function( menu, item, evt )
		if evt == event.click then
			menu:send_item_message( id.route_list, message.move, { page_up = true } )
		end
	end
end

pg_dn_button = function()
	return function( menu, item, evt ) 
		if evt == event.click then
			menu:send_item_message( id.route_list, message.move, { page_down = true } )
		end
	end
end

select_current = function( menu )	
	if playlists.current[index].environment == "random" then
		menu:send_item_message( id.route_list, message.set_selection, { item_id = id.random } )
		menu:send_item_message( id.route_list, message.move, { into_view = true } )
	else
		local env = string.lower( playlists.current[index].environment )
		local lvl = string.lower( playlists.current[index].level )
		local race = string.lower( playlists.current[index].race )
		
		-- print( "looking for", env, lvl, race )
		
		local list = menu:send_item_message( id.route_list, message.get_content, {} )
		local item_count = list:get_item_count()

		for i=1, item_count do
			local item = list:get_item_by_index( i )
			local item_id = item:get_item_id()
			local value = item:send_message( message.get_value, {} )
			
			if item_id ~= id.random then
				if value then
					-- print( "checking", value.environment, value.level, value.race )
					
					if value.environment and value.environment ==  env and value.level and value.level == lvl and value.race and value.race == race then
						local item_id = item:get_item_id()
						if item:is_selectable() then
							menu:send_item_message( id.route_list, message.set_selection, { item_id = item_id } )
							menu:send_item_message( id.route_list, message.move, { into_view = true } )
						end
						local selected_race = menu:send_item_message( id.route_list, message.get_selection )
						if selected_race then
							original_route = selected_race:send_message( message.get_value, {} )
						end
						break
					end
				end
			end
		end
	end
end

do_select = function( menu )
	if new_event then
		menu:send_item_message( id.route_list, message.set_selection, { first = true } )
	else
		select_current( menu )
	end
	
	menu:check_focus()
end

get_route_list = function( data )
	local unlock_level = progress_manager.get_value( "current_level", "default" )
	local is_locked = false
	
	local career_list = {}
	local career_env_list = {}
	local env_count = 0
	
	-- go through the career and build a list of events/environments in the order they are unlocked
	for i=1, #current_career do
		local events = current_career[i]
		
		for j=1, #events do
			local event = events[j]
			local career_environment = string.lower( event.environment )
			
			local found_env = false
			
			for env=1, #career_env_list do
				if career_env_list[env] == career_environment then
					found_env = true
					break
				end
			end
			
			if not found_env then
				env_count = env_count + 1
				career_env_list[env_count] = career_environment
			end
			
			if not career_list[career_environment] then
				career_list[career_environment] = {}
			end

			local found = false

			for k=1, #career_list[career_environment] do
				if 	string.lower( career_list[career_environment][k].environment ) == career_environment and
					string.lower( career_list[career_environment][k].level ) == string.lower( event.level ) and
					string.lower( career_list[career_environment][k].race ) == string.lower( event.race ) then
					found = true
					break
				end
			end
			
			if not found then
				local num_routes = #career_list[career_environment]
				career_list[career_environment][num_routes + 1] = {}
				career_list[career_environment][num_routes + 1].environment = string.lower( event.environment )
				career_list[career_environment][num_routes + 1].level = string.lower( event.level )
				career_list[career_environment][num_routes + 1].race = string.lower( event.race )
				if unlock_level < i then
					career_list[career_environment][num_routes + 1].locked = i -- i is the node, this is when you'll unlock it
					is_locked = false
				end
			end
		end
	end

	-- build the list by environment with routes in order as they are unlocked in the career
	local new_data = {}
	local counter = 1
	
	for career_env, career_routes in pairs( career_list ) do	
		for j=1, #career_routes do
			for k,v in next, data, nil do
				if 	career_routes[j].environment == string.lower( v.environment ) and
					career_routes[j].level == string.lower( v.level ) and 
					career_routes[j].race == string.lower( v.race ) then
					if not new_data[career_routes[j].environment] then
						new_data[career_routes[j].environment] = {}
					end
					if not Application:IsCarMODgeddonEnabled() then
						 v.locked = false -- career_routes[j].locked
					end
					
					new_data[career_routes[j].environment][counter] = v
					data[k] = nil
					counter = counter + 1
					break
				end
			end
		end

		counter = 1
	end

	-- add any routes that don't appear in the career
	if Application:IsCarMODgeddonEnabled() or Application:IsCarMODgeddonEnabled() == false  then
		for k, v in pairs( data ) do
			--print( v.environment, v.level, v.race )
			
			if not new_data[v.environment] then
				new_data[v.environment] = {}
			end
			
			local count = #new_data[v.environment]
			new_data[v.environment][count + 1] = {}
			new_data[v.environment][count + 1] = v
		end
		
		-- CarMODgeddon
		-- Don't organize based on career events, as modded data may included non-career 'environments'
		-- Don't re-order for locked entries, because there aren't any! :)
		local return_data = {}
		for k, v in pairs( new_data ) do
			for j=1, #v do
				return_data[counter] = v[j]
				counter = counter + 1
			end
		end
		return return_data
	end
	
	-- restructure the list by index
	local return_data = {}
	local counter = 1
	
	for i=1, #career_env_list do
		for k, v in pairs( new_data ) do
			if k == career_env_list[i] then
				for j=1, #v do
					return_data[counter] = v[j]
					counter = counter + 1
				end
				break
			end
		end
	end

	-- move locked routes to the bottom of the list in a new section
	if is_locked then
		local locked = {}
		local unlocked = {}
		local locked_counter = 1
		local unlocked_counter = 1

		for i=1, #return_data do
			if return_data[i].locked then
				locked[locked_counter] = return_data[i]
				locked_counter = locked_counter + 1
			else
				unlocked[unlocked_counter] = return_data[i]
				unlocked_counter = unlocked_counter + 1
			end
		end

		return_data = {}

		for i=1, #unlocked do
			return_data[i] = unlocked[i]
		end

		for i=1, #locked do
			return_data[#unlocked + i] = locked[i]
		end
	end
	
	return return_data
end

populate_grid = function( menu )
	local new_grid_items = {}
	local current_env = nil
	
	local route_list = race_config:get_config()
	route_list = get_route_list( route_list )
	
	local first_locked
	
	for i=1, #route_list do
		if route_list[i].locked then
			first_locked = i
			break
		end
	end

	local random = {}
	random.environment	= "random"
	random.level 		= "random"
	random.race 		= "random"
	random.name			= "random"
	--random.writeup	= "random"
	random.image 		= "race\\random_route"
	random.locked		= false
	
	new_grid_items[#new_grid_items+1] = info_header_paged { text=txt.fe_random_ucase, }
	new_grid_items[#new_grid_items+1] = button_value_small_paged{ id=id.random, value=random, group=id.level_group, text=txt.ui_fe_random_route_ucase, locked = false, skin="button_menu.bin",}	
	
	
	local stream_download_progress = Application:StreamingDownloadProgress()
	
	local stream_download_in_progress = false
	if stream_download_progress > 0 and stream_download_progress < 100  then
		stream_download_in_progress = true
	end
	
	for i=1, #route_list do
		local value = {}
		
		value.environment	= route_list[i].environment
		value.level 		= route_list[i].level
		value.race 			= route_list[i].race
		value.name			= route_list[i].name
		--value.writeup 	= route_list[i].writeup
		value.image 		= route_list[i].image
		value.locked		= route_list[i].locked
		

		if stream_download_in_progress then

--			local computed_zad_name = "lev_"..value.environment.."_"..value.level..".zad"
			local computed_zad_name = "lev_"..value.level..".zad"

			local is_present  = Application:IsZadDownloaded(computed_zad_name)

			if not is_present then
				value.download_in_progress = true
			else
				value.download_in_progress = false
			end

		end

		
		if not value.locked then
			if value.environment ~= current_env then
				local environment = EnvironmentConfig[value.environment].name
				new_grid_items[#new_grid_items+1] = info_header_paged { text=environment, }
				current_env = value.environment
			end
		else
			if i == first_locked then
				new_grid_items[#new_grid_items+1] = info_header_paged { text=txt.ui_locked_ucase, }
			end
		end
		
		if value.locked then
			new_grid_items[#new_grid_items+1] = button_value_small_paged{ id=id["level_"..#new_grid_items+1], value=value, group=id.level_group, text=value.name, locked = value.locked, download_in_progress = value.download_in_progress, skin="button_menu_unavailable", }--highlights=false, }
		else
			--print( "unlocked", value.environment, value.level, value.race )
			new_grid_items[#new_grid_items+1] = button_value_small_paged{ id=id["level_"..#new_grid_items+1], value=value, group=id.level_group, text=value.name, locked = value.locked, download_in_progress = value.download_in_progress, skin="button_menu.bin", }
		end
	end
	
	-- for i=1, 200 do
		-- if i%15 == 1 then
			-- new_grid_items[i] = info_header_paged { --[[debug_view=true,]] text="ENVIRONMENT", x=80, player_y=-2 }
		-- else
			-- new_grid_items[i] = button_value_small_paged{ --[[debug_view=true,]] id=id["test"], value=nil, group=id.level_group, text="LEVEL "..i, }
		-- end
	-- end
	
	menu:send_item_message( id.route_list, message.set_new_content, { content = new_grid_items, stripey = true } )
	
	do_select( menu )
end

on_refresh = function()
	return function( menu )
		populate_grid( menu )
	end
end

on_push = function()
	return function( menu )
		route_selected = nil
		last_lockedness = nil
		if not menu:get_item_by_id( id.timer ) then
			menu:add_items( {timer_item { id=id.timer }} )
		end
		event_added = false
		menu:refresh()
	end
end

on_pop = function()
	return function( menu )
		if last_lockedness then
			setup_menu:send_message( message.set_value, { play_anim=true, state="deselect_locked" } )
			last_lockedness = nil
		end
		ticker_manager:release_message( menu.ticker_message_desc )
		menu:get_item_by_id( id.timer ).timer:cancel()

		setup_menu:send_message( message.set_value, { download_in_progress =  false } )

		if event_added then
			local env = string.lower( playlists.current[index].environment )
			local lvl = string.lower( playlists.current[index].level )
			local race = string.lower( playlists.current[index].race )
			
			local image
			-- local tag = race_config :get_tag( env, lvl, race )
			-- local name = tag=="" and "" or txt[tag]
			
			if playlists.current[index].environment == "random" then
				image = "race\\random_route"
			else
				image = race_config:get_image( env, lvl, race )
			end
			
			setup_menu:send_message( message.set_value, { change_image = true, race_thumbnail = image or true, level_name = name } )
			setup_menu:send_message( message.set_value, { play_anim = true, --[[new_event = true]] } )
		end
		if not route_selected and original_route then
			if playlists.current[index].environment == "random" then
				setup_menu:send_message( message.set_value, { change_image = true, race_thumbnail = "race\\random_route", } )
			else
				setup_menu:send_message( message.set_value, { change_image = true, race_thumbnail = original_route.image, --[[level_name = original_route.name]] } )
			end
		end
	end
end

set_button_bar = function( menu )
	local button_y
	if properties.in_network_session then
		button_y = false
	else
		button_y = txt.ui_fe_my_stats_ucase
	end
	menu:send_item_message( id.button_bar, message.set_value, {Y=button_y} )
end

on_focus = function()
	return function( menu, is_focus )
		if is_focus then
			set_button_bar( menu )
		end
	end
end

on_message = function()
	return function( menu, msg_id, parms )
		if msg_id == message.set_value then
			if parms.new_event then
				new_event = parms.new_event
			else
				new_event = nil
			end
			
			if parms.index then
				index = parms.index
			end
			if parms.setup_menu then
				setup_menu = parms.setup_menu
			end
		elseif msg_id == message.timer_expired then
			if parms.evt_data == event_spamming_stopped then
				change_ticker( menu )
			end
		end
	end
end






