module( ..., lube.menu_logic )

local f = require "functions.functions"

local background_tiled	= require "frontend.background_tiled"
local arcade_setup 	= require "frontend.arcade_setup"
local options 		= require "frontend.options"
local stuff			= require "frontend.stuff"
local garage		= require "frontend.garage"
local profile		= require "frontend.profile"

local dlg_yes_no 	= require "dialogue_boxes.yes_no"
local dlg_ok		= require "dialogue_boxes.ok"
local lb_man		= require "frontend.leaderboard_manager"	-- force require here

local progress_manager = require "frontend.progress_manager"
local playlist_setup 		= require "frontend.playlist_setup"

on_event[id.a_button]		= lube.on_click_select()
on_event[id.b_button]		= lube.on_click_pop()
on_event[id.y_button]		= lube.on_click_push( profile )

on_event[id.arcade] = function ( menu, item, evt )
	if evt == event.click then
		ui:push( arcade_setup )
	elseif evt == event.locked_click then
		dlg_ok:send_message ( message.set_value, { title=txt.ui_locked_ucase, body=txt.ui_fe_freeplay_locked_ucase, } )
		ui:push( dlg_ok )
	end
end

on_event[id.garage] = playlist_setup.x_button( "career" )

on_event[id.options] 		= lube.on_click_push( options )
on_event[id.stuff]			= lube.on_click_push( stuff )

function on_initialised( menu, parms )
	menu.do_emergency_exit = do_emergency_exit
end

on_event[id.url] = function ( menu, item, evt )
	if evt == event.click then
		interface:open_browser( "http://www.carmageddon.com/cr" )	-- total hack
	end
end

local career_setup 		= require "frontend.career_setup"
local career_lobby		= require "frontend.career_lobby"
local progress_manager 	= require "frontend.progress_manager"
local multiplayer		= require "frontend.multiplayer"

on_event[id.career] 		= function ( menu, item, evt )
	if evt == event.click then		
		if properties.difficulty_set then
			if setup:interactive_tutorial_done() then
				ui:push( career_setup )
			else
				career_setup:send_message( message.set_value, { do_setup = true } )
				career_lobby:send_message( message.set_value, { do_setup = true } )
			end
		else
			local driver_select = require "frontend.first_time_driver_select"
			ui:push( driver_select )
		end
	end
end

on_event[id.multiplayer] = function ( menu, item, evt )
	if evt == event.click then
		ui:push( multiplayer )
	elseif evt == event.locked_click then
		dlg_ok:send_message ( message.set_value, { title=txt.ui_fe_modified_warning_title_ucase, body=txt.ui_fe_modified_warning_body_ucase, } )
		ui:push( dlg_ok )
	end
end

on_event[id.exit] = function( menu, item, evt )
	if evt == event.click then	
		ui:push( dlg_yes_no )
		dlg_yes_no:send_message( message.set_value, { title=txt.ui_fe_title_exit_game_ucase, body=txt.ui_fe_body_exit_game_ucase, yes_func = endgame_yes_callback } )
	end
end

on_push = function( menu )
	if Application:IsCarMODgeddonEnabled() then
		menu:select_item_by_id( id.arcade )
	else
		menu:select_item_by_id( id.career )
	end

	lube.push_unique( far_stack, background_tiled )
	
	-- dbg_break()
	-- local total_slots = save_manager:get_num_slots()
	-- for i=1, total_slots do
		-- progress_manager.create_career( "default", i )
	-- end
	
	-- progress_manager.create_career( "default" )
	
	local url = menu:get_item_by_id( id.url )
	
	if url then
		if properties.steam_online_enabled then
			url:enable()
		else
			url:disable()
		end
	end
	
	if progress_manager.notifications( menu ) then
		ticker_token = ticker_manager:add_messages( false, false, txt.ui_fe_career_unplayed_events_ticker, 4 )
	end

	properties.stealworthy_car = false
	
	menu:refresh()
	
	properties["start_screen_visible"] = false
end

on_focus = function( menu, is_focus )
	rich_presence:set_next_presence(1)
	properties.vehicle_group = "easy"
end

on_pop = function( menu )
	properties.use_last_visited_event = false
	if ticker_token then
		ticker_manager:release_message( ticker_token )
		ticker_token = nil
	end
end

on_refresh = function( menu )
	if Application:IsCarMODgeddonEnabled() then
		--menu:send_item_message( id.career, message.set_value, { locked = true, skin = "button_menu" } )
		menu:send_item_message( id.arcade, message.set_value, { locked = false, skin = "button_menu" } )
		menu:send_item_message( id.multiplayer, message.set_value, { locked = false, skin = "button_menu" } )
		menu:send_item_message( id.career, message.set_text, { highlight = true } )
		menu:send_item_message( id.arcade, message.set_text, { highlight = true } )
		menu:send_item_message( id.multiplayer, message.set_text, { highlight = true } )
	elseif setup:interactive_tutorial_done() then
		menu:send_item_message( id.arcade, message.set_value, { locked = false, skin = "button_menu" } )
		menu:send_item_message( id.garage, message.set_value, { locked = false, skin = "button_menu" } )
		--menu:send_item_message( id.multiplayer, message.set_value, { locked = false, skin = "button_menu" } )
		menu:send_item_message( id.arcade, message.set_text, { highlight = true } )
		menu:send_item_message( id.garage, message.set_text, { highlight = true } )
		-- menu:send_item_message( id.multiplayer, message.set_text, { highlight = true } )
	else
		menu:send_item_message( id.arcade, message.set_value, { locked = false, skin = "button_menu" } )
		menu:send_item_message( id.garage, message.set_value, { locked = false, skin = "button_menu" } )
		--menu:send_item_message( id.multiplayer, message.set_value, { locked = true, skin = "button_menu" } )
		menu:send_item_message( id.arcade, message.set_text, { highlight = true } )
		menu:send_item_message( id.garage, message.set_text, { highlight = true } )
		-- menu:send_item_message( id.multiplayer, message.set_text, { highlight = true } )
	end
	
	local stream_progress = Application:StreamingDownloadProgress()

	if setup:is_modified() then
		dlg_ok:send_message ( message.set_value, { title=txt.ui_fe_modified_warning_title_ucase, body=txt.ui_fe_modified_warning_body_ucase, } )
		ui:push( dlg_ok )
		menu:send_item_message( id.multiplayer, message.set_value, { locked = false, skin = "button_menu" } )
		menu:send_item_message( id.multiplayer, message.set_text, { highlight = true } )
	elseif stream_progress > 0 and stream_progress < 100 then
		menu:send_item_message( id.multiplayer, message.set_value, { locked = false, skin = "button_menu" } )
		menu:send_item_message( id.multiplayer, message.set_text, { highlight = true } )
	else
		menu:send_item_message( id.multiplayer, message.set_value, { locked = false, skin = "button_menu" } )
		menu:send_item_message( id.multiplayer, message.set_text, { highlight = true } )
	end
end

on_message = function( menu, msg_id, parms )
	if msg_id == message.reset then
		if parms.background then
			menu:send_item_message( id.background, msg_id )
		end
	end
end

endgame_yes_callback = function()
	Application:Quit()
end

do_emergency_exit = function (menu, direction)
	if direction == emergency_exit_direction.network_lobby then
		ui:push( multiplayer )
	elseif direction == emergency_exit_direction.main_menu then
		call_backs.reset_emergency_exit()
	end
end

on_stream_download_in_progress = function(menu)
	local stream_progress = Application:StreamingDownloadProgress()
	if stream_progress >= 100 then
		menu:send_item_message( id.multiplayer, message.set_value, { locked = false, skin = "button_menu" } )
		menu:send_item_message( id.multiplayer, message.set_text, { highlight = true } )
	end	
end

add_item( watch{ handler = on_stream_download_in_progress, 	property_watch="stream_download_in_progress" 	} )
