module( ..., lube.menu_logic )
list = require "lube.menu.list"

local background					= require "frontend.background"
local dlg_ok 						= require "dialogue_boxes.ok"
local main_menu						= require "frontend.main_menu"
local driver_select 				= require "frontend.first_time_driver_select"
local dlg_yes_no 					= require "dialogue_boxes.yes_no"
local f 							= require "functions.functions"
local go_arcade 					= require "logic.control.arcade_race_done_push_ui"
local go_career						= require "logic.control.career_race_done_push_ui"
local go_lan						= require "logic.control.lan_race_done_push_ui"
local go_online 					= require "logic.control.online_race_done_push_ui"
local go_interactive_tutorial		= require "logic.control.interactive_tutorial_done_push_ui"
local go_arcade_abort 				= require "logic.control.arcade_race_abort_push_ui"
local go_career_abort 				= require "logic.control.career_race_abort_push_ui"
local go_lan_abort 					= require "logic.control.lan_race_abort_push_ui"
local go_online_abort 				= require "logic.control.online_race_abort_push_ui"
local go_interactive_tutorial_abort	= require "logic.control.interactive_tutorial_abort_push_ui"
local auto_save						= require "frontend.auto_save_warning"
local progress_manager 				= require "frontend.progress_manager"

local start_menu
local updates_checked

on_message = function( menu, msg_id, msg_parms )
end

function on_initialised( menu, parms )
	menu.do_emergency_exit = do_emergency_exit
end

on_push = function( menu )
	Application:ShowLogInScreen()
	options:unblock_music()

	f.push_all_stacks()
	
	local go = properties["menu_return"]
	if go ~= nil then
		if go == "arcade" then
			go_arcade.run()
		elseif go == "career" then
			go_career.run()
		elseif go == "lan" then
			go_lan.run()
		elseif go == "online" then
			go_online.run()
		elseif go == "interactive_tutorial" then
			go_interactive_tutorial.run()
		elseif go == "arcade_abort" then
			go_arcade_abort.run()
		elseif go == "career_abort" then
			go_career_abort.run()
		elseif go == "lan_abort" then
			go_lan_abort.run()
		elseif go == "online_abort" then
			go_online_abort.run()
		elseif go == "interactive_tutorial_abort" then
			go_interactive_tutorial_abort.run()
		end
	end
	
	progress_manager.create_career( "default" )
	
	updates_checked = false
	check_updates( menu )
end

check_updates = function( menu )
	if properties.show_settings_warning then
		properties.show_settings_warning = false
		ui:push( dlg_ok )
		dlg_ok:send_message( message.set_value, { title=txt.ui_fe_settings_title_ucase, body=txt.ui_fe_settings_body_ucase, ok_func = ok_callback } )
		return
	end
	
	-- if not properties.seen_update_1 then
		-- properties.seen_update_1 = true
		-- ui:push( dlg_ok )
		-- dlg_ok:send_message( message.set_value, { title=txt.ui_fe_update_1_title_ucase, body=txt.ui_fe_update_1_body_ucase, ok_func = ok_callback } )
		-- return
	-- end
	
	-- if not properties.seen_update_2 then
		-- properties.seen_update_2 = true
		-- ui:push( dlg_ok )
		-- dlg_ok:send_message( message.set_value, { title=txt.ui_fe_update_2_title_ucase, body=txt.ui_fe_update_2_body_ucase, ok_func = ok_callback } )
		-- return
	-- end
	
	updates_checked = true
end

ok_callback = function()
	save_manager:save()
	ui:pop()
end

on_update = function( menu )

end

on_focus = function( menu, is_focus )
	rich_presence:set_next_presence(1)
	if is_focus then
		properties["start_screen_visible"] = true
		if not updates_checked then
			check_updates( menu )
			return
		end
		menu:select_item_by_id( id.start )
		save_manager:validate_used_slots()
	end
end

on_event[id.back] = function( menu, item, evt )
	if evt == event.click then
		local start = menu:get_item_by_id( id.start )
		start:hide()
		start_menu = menu
		background:send_message( message.play, { animation = "disabled" } )
		ui:push( dlg_yes_no )
		dlg_yes_no:send_message( message.set_value, { title=txt.ui_fe_title_exit_game_ucase, body=txt.ui_fe_body_exit_game_ucase, yes_func = endgame_yes_callback, no_func = endgame_no_callback } )
	end
end

endgame_yes_callback = function()
	Application:Quit()
end

endgame_no_callback = function()
	local start = start_menu:get_item_by_id( id.start )
	start:show()
	background:send_message( message.play, { animation = "onPush" } )
	ui:pop()
end

on_event[id.start] = function( menu, item, evt )
	if evt == event.click then
		properties["number_player_cars"] = 1
		properties["number_ai_cars"] = 16 - properties["number_player_cars"]
		properties["peds"] = true
		properties["started_new_session"] = true	--Used to increment number of sessions when a race begins loading. If this is true, race will increment session count and set to false.
		if Application:IsSavingEnabled() then
			ui:push( auto_save )
		else
			ui:push( main_menu )
		end
	end
end

update_message = function ( menu, item_id, display_text ) 
	local item = menu:get_item_by_id( item_id )
	if item then 
		if display_text then 
			item:set_alpha( 255 )
			item:send_message( message.set_text, {text=display_text  } )
		else
			item:set_alpha( 0, 0.25 )
		end
	end
end

do_emergency_exit = function (menu, direction)
	if direction == emergency_exit_direction.network_lobby or direction == emergency_exit_direction.main_menu then
		properties["number_player_cars"] = 1
		properties["number_ai_cars"] = 6 - properties["number_player_cars"]
		properties["peds"] = true
		
		properties.player1_is_boy = true
		properties.player1_driver = "DRIVER_BODY"
		car_lot:set_player_car( 1, 1, "eagle_r", "default" )
		car_lot:set_player_car( 1, 2, "eagle_r", "default" )
			
		properties.difficulty = "normal"
		properties.difficulty_set = true
			
		ui:push( main_menu )
	end
end

on_event[id.x_button] = function ( menu, item, evt )
	if evt == event.click then 
	end
end

add_item(
	watch{ handler = function( menu ) update_message ( menu, id.version_message, 	properties.version_message ) end, 		property_watch="version_message" 	}
)

